/* Scrutari - Copyright (c) 2009-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets;

import fr.exemole.sctserver.api.SctEngine;
import fr.exemole.sctserver.request.json.JsonProducerFactory;
import fr.exemole.sctserver.servlets.admin.AdminResponseHandlerFactory;
import fr.exemole.sctserver.servlets.export.ExportResponseHandlerFactory;
import fr.exemole.sctserver.servlets.feed.FeedResponseHandlerFactory;
import fr.exemole.sctserver.servlets.webapp.WebappResponseHandlerFactory;
import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.servlets.HttpServletRequestMap;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.handlers.JsonResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariServlet extends HttpServlet {

    @Override
    public void doGet(HttpServletRequest requete, HttpServletResponse reponse) throws IOException, ServletException {
        process(requete, reponse);
    }

    @Override
    public void doPost(HttpServletRequest requete, HttpServletResponse reponse) throws IOException, ServletException {
        process(requete, reponse);
    }

    private void process(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        String pathInfo = request.getPathInfo();
        if (pathInfo == null) {
            pathInfo = "/";
        }
        int idx = pathInfo.indexOf('/', 2);
        if (idx != -1) {
            String engineName = pathInfo.substring(1, idx);
            String enginePathInfo = pathInfo.substring(idx + 1);
            processEngineRequest(engineName, enginePathInfo, request, response);
        } else {
            processWebappsRequest(pathInfo, request, response);
        }
    }

    private void processWebappsRequest(String path, HttpServletRequest request, HttpServletResponse response) throws IOException {
        ScrutariWebapp scrutariWebapp = ScrutariWebapp.getScrutariWebapp(getServletContext());
        StringBuffer requestURLString = request.getRequestURL();
        if (path.length() == 1) {
            if (requestURLString.charAt(requestURLString.length() - 1) != '/') {
                response.sendRedirect(requestURLString.append('/').toString());
                return;
            }
        }
        ResponseHandler responseHandler = null;
        if (path.endsWith(".css")) {
            responseHandler = ResourceResponseHandlerFactory.getCssHandler(path);
        } else if (path.endsWith(".png")) {
            responseHandler = ResourceResponseHandlerFactory.getPngHandler(path);
        } else {
            RequestMap requestMap = new HttpServletRequestMap(request);
            responseHandler = WebappResponseHandlerFactory.getHandler(path, scrutariWebapp, requestMap);
            if (responseHandler == null) {
                if (scrutariWebapp.getEngine(path) != null) {
                    response.sendRedirect(requestURLString.append('/').toString());
                    return;
                }
            }
        }
        handleResponse(request, response, responseHandler);
    }

    private void processEngineRequest(String engineName, String enginePathInfo, HttpServletRequest request, HttpServletResponse response) throws IOException {
        ScrutariWebapp scrutariWebapp = ScrutariWebapp.getScrutariWebapp(getServletContext());
        SctEngine engine = scrutariWebapp.getEngine(engineName);
        if (engine == null) {
            response.sendError(HttpServletResponse.SC_NOT_FOUND, request.getPathInfo());
            return;
        }
        if (enginePathInfo.length() == 0) {
            response.sendError(HttpServletResponse.SC_FORBIDDEN);
            return;
        }
        RequestMap requestMap = new HttpServletRequestMap(request);
        ResponseHandler responseHandler = null;
        int idx = enginePathInfo.indexOf('/');
        if (idx == -1) {
            String minuscule = enginePathInfo.toLowerCase();
            if (minuscule.equals("json")) {
                JsonProducer jsonProducer = JsonProducerFactory.getJsonProducer(engine.getScrutariSession(), requestMap);
                JsonResponseHandler jsonResponseHandler = new JsonResponseHandler(jsonProducer, requestMap.getParameter("callback"));
                jsonResponseHandler.allowsAllOrigin();
                responseHandler = jsonResponseHandler;
            }
        } else {
            String servletPath = enginePathInfo.substring(0, idx);
            enginePathInfo = enginePathInfo.substring(idx);
            if (servletPath.equals("admin")) {
                responseHandler = AdminResponseHandlerFactory.getHandler(engine, enginePathInfo, requestMap);
            } else if (servletPath.equals("feed")) {
                responseHandler = FeedResponseHandlerFactory.getHandler(engine, enginePathInfo, requestMap);
            } else if (servletPath.equals("css")) {
                responseHandler = ResourceResponseHandlerFactory.getCssHandler(enginePathInfo);
            } else if (servletPath.equals("js")) {
                responseHandler = ResourceResponseHandlerFactory.getJsHandler(enginePathInfo);
            } else if (servletPath.equals("logs")) {
                responseHandler = ResourceResponseHandlerFactory.getLogsHandler(enginePathInfo, engine.getEngineStorage().getLogStorage());
            } else if (servletPath.equals("export")) {
                responseHandler = ExportResponseHandlerFactory.getHandler(engine, enginePathInfo, requestMap);
            }
        }
        handleResponse(request, response, responseHandler);
    }

    private void handleResponse(HttpServletRequest request, HttpServletResponse response, ResponseHandler responseHandler) throws IOException {
        if (responseHandler == null) {
            response.sendError(HttpServletResponse.SC_NOT_FOUND, request.getPathInfo());
            return;
        }
        long lastModified = responseHandler.getLastModified();
        if (lastModified == ResponseHandler.LASTMODIFIED_NOCACHE) {
            response.setHeader("Cache-Control", "no-store");
        } else if (lastModified > 0) {
            response.setDateHeader("Last-Modified", lastModified);
            try {
                long since = request.getDateHeader("If-Modified-Since");
                if (since >= lastModified) {
                    response.setStatus(HttpServletResponse.SC_NOT_MODIFIED);
                    return;
                }
            } catch (IllegalArgumentException iae) {
            }
        }
        responseHandler.handleResponse(response);
    }

}
