/* ScrutariLib_DB - Copyright (c) 2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.db.tools;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.util.exceptions.ImplementationException;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;
import net.scrutari.data.URIPrimitivesUtils;
import net.scrutari.datauri.BaseURI;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.FicheURI;
import net.scrutari.datauri.MotcleURI;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.ScrutariDataURIChecker;
import net.scrutari.datauri.ThesaurusURI;


/**
 *
 * @author Vincent Calame
 */
public class AliasManager implements ScrutariDataURIChecker {

    private final Map<String, BaseURI> mainMap = new HashMap<String, BaseURI>();
    private final Map<BaseURI, String> baseURIMap = new HashMap<BaseURI, String>();

    public AliasManager() {

    }

    public BaseURI getMainBaseURI(String sourceName) {
        return mainMap.get(sourceName);
    }

    public String getSourceName(BaseURI baseURI) {
        return baseURIMap.get(baseURI);
    }

    @Override
    public ScrutariDataURI checkURI(ScrutariDataURI uri) {
        BaseURI baseURI = uri.getBaseURI();
        String sourceName = baseURIMap.get(baseURI);
        if (sourceName == null) {
            return uri;
        }
        BaseURI mainURI = mainMap.get(sourceName);
        if (mainURI == null) {
            return uri;
        }
        if (mainURI.equals(baseURI)) {
            return uri;
        }
        switch (uri.getType()) {
            case ScrutariDataURI.BASEURI_TYPE:
                return mainURI;
            case ScrutariDataURI.CORPUSURI_TYPE:
                return new CorpusURI(mainURI, ((CorpusURI) uri).getCorpusName());
            case ScrutariDataURI.THESAURUSURI_TYPE:
                return new ThesaurusURI(mainURI, ((ThesaurusURI) uri).getThesaurusName());
            case ScrutariDataURI.FICHEURI_TYPE:
                FicheURI ficheURI = (FicheURI) uri;
                return new FicheURI(new CorpusURI(mainURI, ficheURI.getCorpusURI().getCorpusName()), ficheURI.getFicheId());
            case ScrutariDataURI.MOTCLEURI_TYPE:
                MotcleURI motcleURI = (MotcleURI) uri;
                return new MotcleURI(new ThesaurusURI(mainURI, motcleURI.getThesaurusURI().getThesaurusName()), motcleURI.getMotcleId());
            default:
                throw new ImplementationException("Unknown implementation: " + uri.getClass().getName());
        }
    }


    public static AliasManager fromPrimitives(PrimitivesReader primitivesReader) throws IOException {
        AliasManager aliasManager = new AliasManager();
        int mainSize = primitivesReader.readInt();
        for (int i = 0; i < mainSize; i++) {
            String sourceName = primitivesReader.readString();
            BaseURI baseURI = (BaseURI) URIPrimitivesUtils.readScrutariDataURI(primitivesReader);
            aliasManager.mainMap.put(sourceName, baseURI);
        }
        int aliasSize = primitivesReader.readInt();
        for (int i = 0; i < aliasSize; i++) {
            BaseURI baseURI = (BaseURI) URIPrimitivesUtils.readScrutariDataURI(primitivesReader);
            String sourceName = primitivesReader.readString();
            aliasManager.baseURIMap.put(baseURI, sourceName);
        }
        return aliasManager;
    }

    public static void toPrimitives(AliasManager aliasManager, PrimitivesWriter primitivesWriter) throws IOException {
        primitivesWriter.writeInt(aliasManager.mainMap.size());
        for (Map.Entry<String, BaseURI> entry : aliasManager.mainMap.entrySet()) {
            primitivesWriter.writeString(entry.getKey());
            URIPrimitivesUtils.writeScrutariDataURI(entry.getValue(), primitivesWriter);
        }
        primitivesWriter.writeInt(aliasManager.baseURIMap.size());
        for (Map.Entry<BaseURI, String> entry : aliasManager.baseURIMap.entrySet()) {
            URIPrimitivesUtils.writeScrutariDataURI(entry.getKey(), primitivesWriter);
            primitivesWriter.writeString(entry.getValue());
        }
    }


    public static class Builder {

        private final AliasManager aliasManager;

        public Builder() {
            this.aliasManager = new AliasManager();
        }

        public void putMainBaseURI(String sourceName, BaseURI baseURI) {
            aliasManager.mainMap.put(sourceName, baseURI);
            aliasManager.baseURIMap.put(baseURI, sourceName);
        }

        public void putAliasBaseURI(String sourceName, BaseURI baseURI) {
            aliasManager.baseURIMap.put(baseURI, sourceName);
        }

        public boolean containsBaseURI(BaseURI baseURI) {
            return aliasManager.baseURIMap.containsKey(baseURI);
        }

        public String getSourceName(BaseURI baseURI) {
            return aliasManager.baseURIMap.get(baseURI);
        }

        public BaseURI getMainBaseURI(String sourceName) {
            return aliasManager.mainMap.get(sourceName);
        }

        public AliasManager toAliasManager() {
            return aliasManager;
        }

    }

}
