/* ScrutariLib_Data - Copyright (c) 2005-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.data;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;
import net.mapeadores.util.text.Label;
import net.scrutari.datauri.MotcleURI;


/**
 *
 * @author Vincent Calame
 */
public final class MotcleData implements Serializable {

    private final MotcleURI motcleURI;
    private final int codemotcle;
    private final int codethesaurus;
    private final int codebase;
    private final Label[] libelleArray;

    private MotcleData(MotcleURI motcleURI, int codemotcle, int codethesaurus, int codebase, Label[] libelleArray) {
        this.motcleURI = motcleURI;
        this.codemotcle = codemotcle;
        this.codethesaurus = codethesaurus;
        this.codebase = codebase;
        this.libelleArray = libelleArray;
    }

    public int getCodeMotcle() {
        return codemotcle;
    }

    public MotcleURI getMotcleURI() {
        return motcleURI;
    }

    public int getCodeThesaurus() {
        return codethesaurus;
    }

    public int getCodeBase() {
        return codebase;
    }

    public int getLibelleCount() {
        return libelleArray.length;
    }

    public Label getLibelleByLang(Lang lang) {
        int length = libelleArray.length;
        for (int i = 0; i < length; i++) {
            Label libelle = libelleArray[i];
            if (libelle.getLang().equals(lang)) {
                return libelle;
            }
        }
        return null;
    }

    public Label getLibelleByIndex(int index) {
        return libelleArray[index];
    }

    public static MotcleData fromPrimitives(PrimitivesReader primitivesReader) throws IOException {
        MotcleURI motcleURI = (MotcleURI) URIPrimitivesUtils.readScrutariDataURI(primitivesReader);
        int codemotcle = primitivesReader.readInt();
        int codethesaurus = primitivesReader.readInt();
        int codebase = primitivesReader.readInt();
        int libelleLength = primitivesReader.readInt();
        Label[] array = new Label[libelleLength];
        for (int i = 0; i < libelleLength; i++) {
            Lang lang = Lang.build(primitivesReader.readString());
            String libelleString = primitivesReader.readString();
            array[i] = DataLibelleBuilder.toLibelle(lang, libelleString);
        }
        return new MotcleData(motcleURI, codemotcle, codethesaurus, codebase, array);
    }

    public static void toPrimitives(MotcleData motcleData, PrimitivesWriter primitivesWriter) throws IOException {
        URIPrimitivesUtils.writeScrutariDataURI(motcleData.getMotcleURI(), primitivesWriter);
        primitivesWriter.writeInt(motcleData.codemotcle);
        primitivesWriter.writeInt(motcleData.codethesaurus);
        primitivesWriter.writeInt(motcleData.codebase);
        int length = motcleData.libelleArray.length;
        primitivesWriter.writeInt(length);
        for (int i = 0; i < length; i++) {
            Label libelle = motcleData.libelleArray[i];
            primitivesWriter.writeString(libelle.getLang().toString());
            primitivesWriter.writeString(libelle.getLabelString());
        }
    }


    public static class Builder {

        private final MotcleURI motcleURI;
        private final List<Label> libelleList = new ArrayList<Label>();

        /**
         * Construit une instance de MotcleBuffer pour le mot-clé d'URI <code>motcleURI</code>.
         */
        public Builder(MotcleURI motcleURI) {
            this.motcleURI = motcleURI;
        }

        /**
         * Retourne l'URI du mot-clé.
         */
        public MotcleURI getMotcleURI() {
            return motcleURI;
        }

        /**
         * Ajoute une instance de l'interface Label.
         * S'il existe déjà un objet Label pour la même que le libellé en argument, ce premier libellé est remplacé.
         */
        public void addLibelle(Label libelle) {
            libelleList.add(libelle);
        }

        /**
         * Construit une instance de MotcleData à partir des données de l'objet MotcleInfoBuffer.
         */
        public MotcleData toMotcleData(int codemotcle, int codethesaurus, int codebase) {
            int size = libelleList.size();
            Label[] libelleArray = new Label[size];
            for (int i = 0; i < size; i++) {
                libelleArray[i] = libelleList.get(i);
            }
            return new MotcleData(motcleURI, codemotcle, codethesaurus, codebase, libelleArray);
        }

    }

}
