/* ScrutariLib_Data - Copyright (c) 2005-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.data;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;
import net.mapeadores.util.text.Label;
import net.scrutari.datauri.ThesaurusURI;


/**
 *
 * @author Vincent Calame
 */
public final class ThesaurusData implements Serializable {

    private final ThesaurusURI thesaurusURI;
    private final int codethesaurus;
    private final int codebase;
    private final ThesaurusLangData[] thesaurusLangDataArray;
    private final int[] codesMotscles;

    private ThesaurusData(ThesaurusURI thesaurusURI, int codethesaurus, int codebase, ThesaurusLangData[] thesaurusLangDataArray, int[] codesMotscles) {
        this.codethesaurus = codethesaurus;
        this.codebase = codebase;
        this.thesaurusLangDataArray = thesaurusLangDataArray;
        this.codesMotscles = codesMotscles;
        this.thesaurusURI = thesaurusURI;
    }

    public ThesaurusURI getThesaurusURI() {
        return thesaurusURI;
    }

    public int getCodeThesaurus() {
        return codethesaurus;
    }

    public int getCodeBase() {
        return codebase;
    }

    public ThesaurusLangData getLangDataByLang(Lang lang) {
        int length = thesaurusLangDataArray.length;
        for (int i = 0; i < length; i++) {
            ThesaurusLangData thesaurusLangData = thesaurusLangDataArray[i];
            if (thesaurusLangData.lang.equals(lang)) {
                return thesaurusLangData;
            }
        }
        return null;
    }

    public int getLangDataCount() {
        return thesaurusLangDataArray.length;
    }

    public ThesaurusLangData getLangDataByIndex(int index) {
        return thesaurusLangDataArray[index];
    }

    public int getMotcleCount() {
        return codesMotscles.length;
    }

    public int getCodeMotcle(int i) {
        return codesMotscles[i];
    }

    public static ThesaurusData fromPrimitives(PrimitivesReader primitivesReader) throws IOException {
        ThesaurusURI thesaurusURI = (ThesaurusURI) URIPrimitivesUtils.readScrutariDataURI(primitivesReader);
        int codethesaurus = primitivesReader.readInt();
        int codebase = primitivesReader.readInt();
        int langDataLength = primitivesReader.readInt();
        ThesaurusLangData[] thesaurusLangDataArray = new ThesaurusLangData[langDataLength];
        for (int i = 0; i < langDataLength; i++) {
            Lang lang = Lang.build(primitivesReader.readString());
            String libelleString = primitivesReader.readString();
            thesaurusLangDataArray[i] = new ThesaurusLangData(DataLibelleBuilder.toLibelle(lang, libelleString));
        }
        int motsclesLength = primitivesReader.readInt();
        int[] codesMotscles = new int[motsclesLength];
        for (int i = 0; i < motsclesLength; i++) {
            codesMotscles[i] = primitivesReader.readInt();
        }
        return new ThesaurusData(thesaurusURI, codethesaurus, codebase, thesaurusLangDataArray, codesMotscles);
    }

    public static void toPrimitives(ThesaurusData thesaurusData, PrimitivesWriter primitivesWriter) throws IOException {
        URIPrimitivesUtils.writeScrutariDataURI(thesaurusData.thesaurusURI, primitivesWriter);
        primitivesWriter.writeInt(thesaurusData.codethesaurus);
        primitivesWriter.writeInt(thesaurusData.codebase);
        int langDataLength = thesaurusData.thesaurusLangDataArray.length;
        primitivesWriter.writeInt(langDataLength);
        for (int i = 0; i < langDataLength; i++) {
            ThesaurusLangData langData = thesaurusData.thesaurusLangDataArray[i];
            primitivesWriter.writeString(langData.lang.toString());
            primitivesWriter.writeString(langData.getLibelle().getLabelString());
        }
        int motsclesLength = thesaurusData.codesMotscles.length;
        primitivesWriter.writeInt(motsclesLength);
        for (int i = 0; i < motsclesLength; i++) {
            primitivesWriter.writeInt(thesaurusData.codesMotscles[i]);
        }
    }


    public static class ThesaurusLangData {

        private final Lang lang;
        private final Label libelle;

        private ThesaurusLangData(Label libelle) {
            this.lang = libelle.getLang();
            this.libelle = libelle;
        }

        public Label getLibelle() {
            return libelle;
        }

        public Lang getLang() {
            return lang;
        }

    }


    public static class Builder {

        private final ThesaurusURI thesaurusURI;
        private final Map<Lang, Label> libelleMap = new LinkedHashMap<Lang, Label>();
        private final List<Integer> motcleList = new ArrayList<Integer>();

        public Builder(ThesaurusURI thesaurusURI) {
            this.thesaurusURI = thesaurusURI;
        }

        public ThesaurusURI getThesaurusURI() {
            return thesaurusURI;
        }

        public void addLibelle(Label libelle) {
            if (libelle == null) {
                throw new IllegalArgumentException("libelle is null");
            }
            libelleMap.put(libelle.getLang(), libelle);
        }

        public void addCodeMotcle(int codemotcle) {
            motcleList.add(codemotcle);
        }

        public ThesaurusData toThesaurusData(int codethesaurus, int codebase) {
            int motcleCount = motcleList.size();
            int[] codesMotscles = new int[motcleCount];
            for (int i = 0; i < motcleCount; i++) {
                codesMotscles[i] = motcleList.get(i);
            }
            ThesaurusLangData[] thesaurusLangDataArray = new ThesaurusLangData[libelleMap.size()];
            int p = 0;
            for (Label libelle : libelleMap.values()) {
                ThesaurusData.ThesaurusLangData thesaurusLangData = new ThesaurusData.ThesaurusLangData(libelle);
                thesaurusLangDataArray[p] = thesaurusLangData;
                p++;
            }
            return new ThesaurusData(thesaurusURI, codethesaurus, codebase, thesaurusLangDataArray, codesMotscles);
        }

    }

}
