/* ScrutariLib_SearchEngine - Copyright (c) 2013-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logicaloperation.LogicalOperationConstants;
import net.mapeadores.util.logicaloperation.Operand;
import net.mapeadores.util.logicaloperation.SubOperand;
import net.scrutari.data.FicheData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.searchengine.api.operands.filters.CodeOperand;
import net.scrutari.searchengine.api.operands.filters.FicheDataOperand;
import net.scrutari.searchengine.api.options.SearchOptionsDef;


/**
 *
 * @author Vincent Calame
 */
public class EligibilityBuffer {

    private final DataAccess dataAccess;
    private final Map<Integer, Boolean> bufferMap = new HashMap<Integer, Boolean>();
    private final Operand ficheEligibilityOperand;
    private final Operand motcleEligibilityOperand;
    private Set<Lang> langCheckSet;
    private FicheData currentFicheData;
    private boolean currentFicheDataInit;

    public EligibilityBuffer(DataAccess dataAccess, SearchOptionsDef searchOptionsDef) {
        this(dataAccess, searchOptionsDef.getFicheEligibilityOperand(), searchOptionsDef.getMotcleEligibilityOperand());
    }

    public EligibilityBuffer(DataAccess dataAccess, Operand ficheEligibilityOperand, Operand motcleEligibilityOperand) {
        this.dataAccess = dataAccess;
        this.ficheEligibilityOperand = ficheEligibilityOperand;
        this.motcleEligibilityOperand = motcleEligibilityOperand;
    }

    public void addLangCheck(Lang[] langArray) {
        langCheckSet = new HashSet<Lang>();
        for (Lang lang : langArray) {
            langCheckSet.add(lang);
        }
    }

    public boolean acceptFiche(int codefiche) {
        if ((ficheEligibilityOperand == null) & (langCheckSet == null)) {
            return true;
        }
        Boolean bool = bufferMap.get(codefiche);
        if (bool != null) {
            return bool;
        }
        boolean bl = checkAcceptCodeFiche(codefiche);
        bufferMap.put(codefiche, bl);
        return bl;
    }

    public boolean acceptMotcle(int codemotcle) {
        if (motcleEligibilityOperand == null) {
            return true;
        }
        Boolean bool = bufferMap.get(codemotcle);
        if (bool != null) {
            return bool;
        }
        boolean bl = checkAcceptCodeMotcle(codemotcle);
        bufferMap.put(codemotcle, bl);
        return bl;
    }

    private void clearCurrent() {
        currentFicheData = null;
        currentFicheDataInit = false;
    }

    private boolean checkAcceptCodeFiche(int code) {
        boolean test = true;
        if (ficheEligibilityOperand != null) {
            test = testOperand(ficheEligibilityOperand, code);
        }
        if ((test) && (langCheckSet != null)) {
            Lang ficheLang;
            if (currentFicheData != null) {
                ficheLang = currentFicheData.getLang();
            } else {
                ficheLang = dataAccess.getFicheLang(code);
            }
            if (!langCheckSet.contains(ficheLang)) {
                test = false;
            }
        }
        clearCurrent();
        return test;
    }

    private boolean checkAcceptCodeMotcle(int code) {
        boolean test = testOperand(motcleEligibilityOperand, code);
        clearCurrent();
        return test;
    }

    private boolean testOperand(Operand operand, int code) {
        if (operand instanceof CodeOperand) {
            return ((CodeOperand) operand).acceptCode(code);
        } else if (operand instanceof FicheDataOperand) {
            return ((FicheDataOperand) operand).acceptFicheData(getFicheData(code));
        } else if (operand instanceof SubOperand) {
            SubOperand subOperand = (SubOperand) operand;
            short operator = subOperand.getOperator();
            if (operator == LogicalOperationConstants.UNION_OPERATOR) {
                return testUnion(subOperand, code);
            } else if (operator == LogicalOperationConstants.INTERSECTION_OPERATOR) {
                return testIntersection(subOperand, code);
            } else {
                throw new SwitchException("Unknown operator = " + operator);
            }
        } else {
            throw new SwitchException("Unknown operand instance = " + operand.getClass().getName());
        }

    }

    private boolean testUnion(SubOperand subOperand, int code) {
        int count = subOperand.getOperandCount();
        for (int i = 0; i < count; i++) {
            Operand operand = subOperand.getOperand(i);
            boolean test = testOperand(operand, code);
            if (test) {
                return true;
            }
        }
        return false;
    }

    private boolean testIntersection(SubOperand subOperand, int code) {
        int count = subOperand.getOperandCount();
        for (int i = 0; i < count; i++) {
            Operand operand = subOperand.getOperand(i);
            boolean test = testOperand(operand, code);
            if (!test) {
                return false;
            }
        }
        return true;
    }

    private FicheData getFicheData(int code) {
        if (!currentFicheDataInit) {
            currentFicheDataInit = true;
            currentFicheData = dataAccess.getFicheDataByCode(code);
        }
        return currentFicheData;
    }

}
