/* ScrutariLib_SearchEngine - Copyright (c) 2014-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.SortedMap;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangList;
import net.mapeadores.util.logicaloperation.Operand;
import net.mapeadores.util.primitives.ByteArrayKey;
import net.mapeadores.util.primitives.io.DataOutputPrimitivesWriter;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.searchengine.api.CanonicalQ;
import net.scrutari.searchengine.api.options.PertinencePonderation;
import net.scrutari.searchengine.api.options.ScrutariDataURIList;
import net.scrutari.searchengine.api.options.SearchOptionsDef;


/**
 *
 * @author Vincent Calame
 */
public final class SearchEngineUtils {

    private SearchEngineUtils() {
    }

    public static ByteArrayKey toKey(CanonicalQ canonicalQ, SearchOptionsDef searchOptionsDef) {
        try {
            ByteArrayOutputStream stream = new ByteArrayOutputStream();
            DataOutputStream dataOutput = new DataOutputStream(stream);
            DataOutputPrimitivesWriter primWriter = new DataOutputPrimitivesWriter(dataOutput);
            primWriter.writeString(canonicalQ.toString());
            writeSearchOptions(searchOptionsDef, primWriter);
            return new ByteArrayKey(stream.toByteArray());
        } catch (IOException ioe) {
            throw new ShouldNotOccurException(ioe);
        }
    }

    private static void writeSearchOptions(SearchOptionsDef searchOptionsDef, DataOutputPrimitivesWriter primWriter) throws IOException {
        if (searchOptionsDef == null) {
            primWriter.writeBoolean(false);
            return;
        }
        primWriter.writeBoolean(true);
        Lang langUi = searchOptionsDef.getLangUi();
        if (langUi != null) {
            primWriter.writeString(langUi.toString());
        } else {
            primWriter.writeString("");
        }
        LangList langList = searchOptionsDef.getFilterLangList();
        int langCount = langList.getLangCount();
        if (langCount > 0) {
            primWriter.writeInt(1);
            primWriter.writeInt(langCount);
            for (int i = 0; i < langCount; i++) {
                primWriter.writeString(langList.getLang(i).toString());
            }
        }
        writeList(2, ScrutariDataURI.BASEURI_TYPE, searchOptionsDef, primWriter);
        writeList(3, ScrutariDataURI.CORPUSURI_TYPE, searchOptionsDef, primWriter);
        writeList(4, ScrutariDataURI.THESAURUSURI_TYPE, searchOptionsDef, primWriter);
        writeList(5, ScrutariDataURI.FICHEURI_TYPE, searchOptionsDef, primWriter);
        writeList(6, ScrutariDataURI.MOTCLEURI_TYPE, searchOptionsDef, primWriter);
        Operand ficheEligibilityOperand = searchOptionsDef.getFicheEligibilityOperand();
        if (ficheEligibilityOperand != null) {
            primWriter.writeInt(7);
            primWriter.writeString(ficheEligibilityOperand.toString());
        }
        Operand motcleEligibilityOperand = searchOptionsDef.getMotcleEligibilityOperand();
        if (motcleEligibilityOperand != null) {
            primWriter.writeInt(8);
            primWriter.writeString(motcleEligibilityOperand.toString());
        }
        PertinencePonderation pertinencePonderation = searchOptionsDef.getPertinencePonderation();
        if (pertinencePonderation != null) {
            primWriter.writeInt(9);
            primWriter.writeInt(toPertinenceInt(pertinencePonderation.getOccurrencePonderation()));
            primWriter.writeInt(toPertinenceInt(pertinencePonderation.getDatePonderation()));
            primWriter.writeInt(toPertinenceInt(pertinencePonderation.getOriginPonderation()));
        }
        int attributeCount = searchOptionsDef.getAttributeCount();
        if (attributeCount > 0) {
            primWriter.writeInt(10);
            primWriter.writeInt(attributeCount);
            if (attributeCount == 1) {
                writeAttribute(searchOptionsDef.getAttributeByIndex(0), primWriter);
            } else {
                SortedMap<String, Attribute> attributeMap = new TreeMap<String, Attribute>();
                for (int i = 0; i < attributeCount; i++) {
                    Attribute attribute = searchOptionsDef.getAttributeByIndex(i);
                    attributeMap.put(attribute.getAttributeKey().toString(), attribute);
                }
                for (Attribute attribute : attributeMap.values()) {
                    writeAttribute(attribute, primWriter);
                }
            }
        }
    }

    private static void writeAttribute(Attribute attribute, DataOutputPrimitivesWriter primWriter) throws IOException {
        primWriter.writeString(attribute.getAttributeKey().toString());
        int valueCount = attribute.getValueCount();
        if (valueCount == 1) {
            primWriter.writeInt(1);
            primWriter.writeString(attribute.getValue(0));
        } else {
            SortedSet<String> valSet = new TreeSet<String>();
            for (int i = 0; i < valueCount; i++) {
                valSet.add(attribute.getValue(i));
            }
            primWriter.writeInt(valSet.size());
            for (String s : valSet) {
                primWriter.writeString(s);
            }
        }
    }

    private static void writeList(int numero, short type, SearchOptionsDef searchOptionsDef, DataOutputPrimitivesWriter primWriter) throws IOException {
        ScrutariDataURIList filterScrutariDataURIList = searchOptionsDef.getFilterScrutariDataURIList(type);
        int count = filterScrutariDataURIList.getScrutariDataURICount();
        if (count > 0) {
            primWriter.writeInt(numero);
            primWriter.writeInt(count);
            for (int i = 0; i < count; i++) {
                primWriter.writeString(filterScrutariDataURIList.getScrutariDataURI(i).toString());
            }
        }
    }

    private static int toPertinenceInt(float f) {
        return (int) (f * 100);
    }

}
