/* ScrutariLib_SearchEngine - Copyright (c) 2014-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.options;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.attr.AppendableAttributeHolderBuilder;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangList;
import net.mapeadores.util.localisation.LangListUtils;
import net.mapeadores.util.logicaloperation.LogicalOperationConstants;
import net.mapeadores.util.logicaloperation.LogicalOperationUtils;
import net.mapeadores.util.logicaloperation.Operand;
import net.mapeadores.util.text.CleanedString;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.FicheData;
import net.scrutari.data.MotcleData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.searchengine.api.options.PertinencePonderation;
import net.scrutari.searchengine.api.options.ScrutariDataURIList;
import net.scrutari.searchengine.api.options.SearchOptionsDef;


/**
 *
 * @author Vincent Calame
 */
public class SearchOptionsDefBuilder {

    public final static SearchOptionsDef EMPTY_SEARCHOPTIONSDEF = new InternalSearchOptionsDef(null, null, new HashMap<Short, ScrutariDataURIList>(), null, null, null, AttributeUtils.EMPTY_ATTRIBUTEHOLDER);
    private final AppendableAttributeHolderBuilder attributeHolderBuilder = new AppendableAttributeHolderBuilder();
    private final List<Operand> ficheOperandList = new ArrayList<Operand>();
    private final List<Operand> motcleOperandList = new ArrayList<Operand>();
    private final Set<ScrutariDataURI> baseUriSet = new LinkedHashSet<ScrutariDataURI>();
    private final Set<ScrutariDataURI> corpusUriSet = new LinkedHashSet<ScrutariDataURI>();
    private final Set<ScrutariDataURI> thesaurusUriSet = new LinkedHashSet<ScrutariDataURI>();
    private final Set<ScrutariDataURI> ficheUriSet = new LinkedHashSet<ScrutariDataURI>();
    private final Set<ScrutariDataURI> motcleUriSet = new LinkedHashSet<ScrutariDataURI>();
    private final Set<Lang> filterLangSet = new LinkedHashSet<Lang>();
    private Lang langUi;
    private PertinencePonderation pertinencePonderation;

    public SearchOptionsDefBuilder() {
    }

    public void setLangUi(Lang langUi) {
        this.langUi = langUi;
    }

    public void addFilterLang(Lang lang) {
        filterLangSet.add(lang);
    }

    public boolean hasFilterLang() {
        return !filterLangSet.isEmpty();
    }

    public Lang reduceToUniqueLang() {
        if (filterLangSet.isEmpty()) {
            return null;
        }
        boolean first = true;
        Lang result = null;
        for (Iterator<Lang> it = filterLangSet.iterator(); it.hasNext();) {
            if (first) {
                result = it.next();
                first = false;
            } else {
                it.next();
                it.remove();
            }
        }
        return result;
    }

    public void setPertinencePonderation(PertinencePonderation pertinencePonderation) {
        this.pertinencePonderation = pertinencePonderation;
    }

    public void addFicheOperand(Operand operand) {
        if (operand != null) {
            ficheOperandList.add(operand);
        }
    }

    public void addMotcleOperand(Operand operand) {
        if (operand != null) {
            motcleOperandList.add(operand);
        }
    }

    public Set<ScrutariDataURI> getURISet(short type) {
        switch (type) {
            case ScrutariDataURI.BASEURI_TYPE:
                return baseUriSet;
            case ScrutariDataURI.CORPUSURI_TYPE:
                return corpusUriSet;
            case ScrutariDataURI.THESAURUSURI_TYPE:
                return thesaurusUriSet;
            case ScrutariDataURI.FICHEURI_TYPE:
                return ficheUriSet;
            case ScrutariDataURI.MOTCLEURI_TYPE:
                return motcleUriSet;
            default:
                throw new SwitchException("Unkown type: " + type);
        }
    }

    public void addScrutariDataURI(ScrutariDataURI scrutariDataURI) {
        switch (scrutariDataURI.getType()) {
            case ScrutariDataURI.BASEURI_TYPE:
                baseUriSet.add(scrutariDataURI);
                break;
            case ScrutariDataURI.CORPUSURI_TYPE:
                corpusUriSet.add(scrutariDataURI);
                break;
            case ScrutariDataURI.THESAURUSURI_TYPE:
                thesaurusUriSet.add(scrutariDataURI);
                break;
            case ScrutariDataURI.FICHEURI_TYPE:
                ficheUriSet.add(scrutariDataURI);
                break;
            case ScrutariDataURI.MOTCLEURI_TYPE:
                motcleUriSet.add(scrutariDataURI);
                break;
            default:
                throw new SwitchException("Unkown type: " + scrutariDataURI.getType());
        }
    }

    public void addScrutariDataURIFromCodeList(DataAccess dataAccess, short type, CodeList codeList) {
        if (codeList == null) {
            return;
        }
        int codeCount = codeList.getCodeCount();
        for (int i = 0; i < codeCount; i++) {
            int code = codeList.getCode(i);
            switch (type) {
                case ScrutariDataURI.BASEURI_TYPE:
                    BaseData baseData = dataAccess.getBaseDataByCode(code);
                    if (baseData != null) {
                        baseUriSet.add(baseData.getBaseURI());
                    }
                    break;
                case ScrutariDataURI.CORPUSURI_TYPE:
                    CorpusData corpusData = dataAccess.getCorpusDataByCode(code);
                    if (corpusData != null) {
                        corpusUriSet.add(corpusData.getCorpusURI());
                    }
                    break;
                case ScrutariDataURI.THESAURUSURI_TYPE:
                    ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(code);
                    if (thesaurusData != null) {
                        thesaurusUriSet.add(thesaurusData.getThesaurusURI());
                    }
                    break;
                case ScrutariDataURI.FICHEURI_TYPE:
                    FicheData ficheData = dataAccess.getFicheDataByCode(code);
                    if (ficheData != null) {
                        ficheUriSet.add(ficheData.getFicheURI());
                    }
                    break;
                case ScrutariDataURI.MOTCLEURI_TYPE:
                    MotcleData motcleData = dataAccess.getMotcleDataByCode(code);
                    if (motcleData != null) {
                        motcleUriSet.add(motcleData.getMotcleURI());
                    }
                    break;
                default:
                    throw new SwitchException("Unkown type: " + type);
            }
        }
    }


    public SearchOptionsDef toSearchOptionsDef() {
        Operand ficheRootOperand = null;
        int ficheOperandSize = ficheOperandList.size();
        if (ficheOperandSize > 0) {
            if (ficheOperandSize == 1) {
                ficheRootOperand = ficheOperandList.get(0);
            } else {
                ficheRootOperand = LogicalOperationUtils.toSubOperand(ficheOperandList.toArray(new Operand[ficheOperandSize]), LogicalOperationConstants.INTERSECTION_OPERATOR);
            }
        }
        Operand motcleRootOperand = null;
        int motcleOperandSize = motcleOperandList.size();
        if (motcleOperandSize > 0) {
            if (motcleOperandSize == 1) {
                motcleRootOperand = motcleOperandList.get(0);
            } else {
                motcleRootOperand = LogicalOperationUtils.toSubOperand(motcleOperandList.toArray(new Operand[motcleOperandSize]), LogicalOperationConstants.INTERSECTION_OPERATOR);
            }
        }
        Map<Short, ScrutariDataURIList> scrutariDataURIListMap = new HashMap<Short, ScrutariDataURIList>();
        if (!baseUriSet.isEmpty()) {
            scrutariDataURIListMap.put(ScrutariDataURI.BASEURI_TYPE, OptionsUtils.toScrutariDataURIList(ScrutariDataURI.BASEURI_TYPE, baseUriSet));
        }
        if (!corpusUriSet.isEmpty()) {
            scrutariDataURIListMap.put(ScrutariDataURI.CORPUSURI_TYPE, OptionsUtils.toScrutariDataURIList(ScrutariDataURI.CORPUSURI_TYPE, corpusUriSet));
        }
        if (!thesaurusUriSet.isEmpty()) {
            scrutariDataURIListMap.put(ScrutariDataURI.THESAURUSURI_TYPE, OptionsUtils.toScrutariDataURIList(ScrutariDataURI.THESAURUSURI_TYPE, thesaurusUriSet));
        }
        if (!ficheUriSet.isEmpty()) {
            scrutariDataURIListMap.put(ScrutariDataURI.FICHEURI_TYPE, OptionsUtils.toScrutariDataURIList(ScrutariDataURI.FICHEURI_TYPE, ficheUriSet));
        }
        if (!motcleUriSet.isEmpty()) {
            scrutariDataURIListMap.put(ScrutariDataURI.MOTCLEURI_TYPE, OptionsUtils.toScrutariDataURIList(ScrutariDataURI.MOTCLEURI_TYPE, motcleUriSet));
        }
        LangList filterLangList;
        if (filterLangSet.isEmpty()) {
            filterLangList = LangListUtils.EMPTY_LANGLIST;
        } else {
            filterLangList = LangListUtils.fromCollection(filterLangSet);
        }
        return new InternalSearchOptionsDef(langUi, filterLangList, scrutariDataURIListMap, ficheRootOperand, motcleRootOperand, pertinencePonderation, attributeHolderBuilder.toAttributeHolder());
    }

    public Attribute getAttribute(AttributeKey attributeKey) {
        return attributeHolderBuilder.getAttribute(attributeKey);
    }

    public void addAttribute(AttributeKey attributeKey, List<CleanedString> valueList) {
        attributeHolderBuilder.addAttribute(attributeKey, valueList);
    }

    public void addAttribute(Attribute attribute) {
        attributeHolderBuilder.addAttribute(attribute);
    }

    public static SearchOptionsDefBuilder derive(SearchOptionsDef searchOptionsDef, boolean withOperand) {
        SearchOptionsDefBuilder searchOptionsDefBuilder = new SearchOptionsDefBuilder();
        searchOptionsDefBuilder.setLangUi(searchOptionsDef.getLangUi());
        LangList filterLangList = searchOptionsDef.getFilterLangList();
        int langCount = filterLangList.getLangCount();
        for (int i = 0; i < langCount; i++) {
            searchOptionsDefBuilder.addFilterLang(filterLangList.getLang(i));
        }
        searchOptionsDefBuilder.setPertinencePonderation(searchOptionsDef.getPertinencePonderation());
        derive(searchOptionsDef, searchOptionsDefBuilder, ScrutariDataURI.BASEURI_TYPE);
        derive(searchOptionsDef, searchOptionsDefBuilder, ScrutariDataURI.CORPUSURI_TYPE);
        derive(searchOptionsDef, searchOptionsDefBuilder, ScrutariDataURI.THESAURUSURI_TYPE);
        derive(searchOptionsDef, searchOptionsDefBuilder, ScrutariDataURI.FICHEURI_TYPE);
        derive(searchOptionsDef, searchOptionsDefBuilder, ScrutariDataURI.MOTCLEURI_TYPE);
        int attributeCount = searchOptionsDef.getAttributeCount();
        for (int i = 0; i < attributeCount; i++) {
            searchOptionsDefBuilder.addAttribute(searchOptionsDef.getAttributeByIndex(i));
        }
        if (withOperand) {
            searchOptionsDefBuilder.addFicheOperand(searchOptionsDef.getFicheEligibilityOperand());
            searchOptionsDefBuilder.addMotcleOperand(searchOptionsDef.getMotcleEligibilityOperand());
        }
        return searchOptionsDefBuilder;
    }

    private static void derive(SearchOptionsDef searchOptionsDef, SearchOptionsDefBuilder searchOptionsDefBuilder, short uriType) {
        ScrutariDataURIList uriList = searchOptionsDef.getFilterScrutariDataURIList(uriType);
        int count = uriList.getScrutariDataURICount();
        for (int i = 0; i < count; i++) {
            searchOptionsDefBuilder.addScrutariDataURI(uriList.getScrutariDataURI(i));
        }
    }


    private static class InternalSearchOptionsDef implements SearchOptionsDef {

        private final Lang langUi;
        private final LangList filterLangList;
        private final Map<Short, ScrutariDataURIList> scrutariDataURIListMap;
        private final Operand ficheEligibilityOperand;
        private final Operand motcleEligibilityOperand;
        private final PertinencePonderation pertinencePonderation;
        private final AttributeHolder attributeHolder;

        private InternalSearchOptionsDef(Lang langUi, LangList filterLangList, Map<Short, ScrutariDataURIList> scrutariDataURIListMap, Operand ficheEligibilityOperand, Operand motcleEligibilityOperand, PertinencePonderation pertinencePonderation, AttributeHolder attributeHolder) {
            this.langUi = langUi;
            this.filterLangList = filterLangList;
            this.ficheEligibilityOperand = ficheEligibilityOperand;
            this.motcleEligibilityOperand = motcleEligibilityOperand;
            this.pertinencePonderation = pertinencePonderation;
            this.scrutariDataURIListMap = scrutariDataURIListMap;
            this.attributeHolder = attributeHolder;
        }

        @Override
        public Lang getLangUi() {
            return langUi;
        }

        @Override
        public LangList getFilterLangList() {
            return filterLangList;
        }

        @Override
        public Operand getFicheEligibilityOperand() {
            return ficheEligibilityOperand;
        }

        @Override
        public Operand getMotcleEligibilityOperand() {
            return motcleEligibilityOperand;
        }

        @Override
        public PertinencePonderation getPertinencePonderation() {
            return pertinencePonderation;
        }

        @Override
        public ScrutariDataURIList getFilterScrutariDataURIList(short type) {
            ScrutariDataURIList list = scrutariDataURIListMap.get(type);
            if (list != null) {
                return list;
            } else {
                return OptionsUtils.getEmptyScrutariDataURIList(type);
            }
        }

        @Override
        public Attribute getAttributeByKey(AttributeKey key) {
            return attributeHolder.getAttributeByKey(key);
        }

        @Override
        public int getAttributeCount() {
            return attributeHolder.getAttributeCount();
        }

        @Override
        public Attribute getAttributeByIndex(int i) {
            return attributeHolder.getAttributeByIndex(i);
        }

    }

}
