/* ScrutariLib_SearchEngine - Copyright (c) 2011-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.result.io;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.ParseException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangList;
import net.mapeadores.util.primitives.io.DataInputPrimitivesReader;
import net.mapeadores.util.primitives.io.DataOutputPrimitivesWriter;
import net.mapeadores.util.primitives.io.PrimitivesReader;
import net.mapeadores.util.primitives.io.PrimitivesWriter;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.searchengine.api.CanonicalQ;
import net.scrutari.searchengine.api.QId;
import net.scrutari.searchengine.api.options.SearchOptionsDef;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.FicheSearchResultGroup;
import net.scrutari.searchengine.api.result.FicheSearchResultInfo;
import net.scrutari.searchengine.api.result.FicheSearchSource;
import net.scrutari.searchengine.api.result.InAlineaOccurrenceList;
import net.scrutari.searchengine.api.result.InLangOccurrenceList;
import net.scrutari.searchengine.api.result.MotcleSearchResultInfo;
import net.scrutari.searchengine.api.result.SearchTokenOccurrence;
import net.scrutari.searchengine.api.result.SearchTokenOccurrenceList;
import net.scrutari.searchengine.tools.options.SearchOptionsDefBuilder;
import net.scrutari.searchengine.tools.result.FicheSearchResultBuilder;
import net.scrutari.searchengine.tools.result.FicheSearchResultGroupBuilder;
import net.scrutari.searchengine.tools.result.FicheSearchResultInfoBuilder;
import net.scrutari.searchengine.tools.result.FicheSearchSourceBuilder;
import net.scrutari.searchengine.tools.result.MotcleSearchResultBuilder;
import net.scrutari.searchengine.tools.result.ResultUtils;
import net.scrutari.searchengine.tools.result.SearchTokenOccurrenceBuilder;


/**
 *
 * @author Vincent Calame
 */
public final class FicheSearchResultStorage {

    private final static int CACHE_VERSION = 1;

    private FicheSearchResultStorage() {
    }

    public static void write(File file, FicheSearchResult ficheSearchResult) throws IOException {
        DataOutputStream dataOutput = new DataOutputStream(new FileOutputStream(file));
        DataOutputPrimitivesWriter primWriter = new DataOutputPrimitivesWriter(dataOutput);
        FicheSearchResultStorage.write(primWriter, ficheSearchResult);
        dataOutput.close();
    }

    public static void write(PrimitivesWriter primitivesWriter, FicheSearchResult ficheSearchResult) throws IOException {
        primitivesWriter.writeInt(CACHE_VERSION);
        writeFicheSearchSource(primitivesWriter, ficheSearchResult.getFicheSearchSource());
        primitivesWriter.writeInt(ficheSearchResult.getFicheCount());
        primitivesWriter.writeInt(ficheSearchResult.getFicheMaximum());
        int motcleCount = ficheSearchResult.getMotcleCount();
        primitivesWriter.writeInt(motcleCount);
        CodeList codecorpusList = ficheSearchResult.getCodecorpusList();
        int corpusCount = codecorpusList.getCodeCount();
        primitivesWriter.writeInt(corpusCount);
        for (int i = 0; i < corpusCount; i++) {
            primitivesWriter.writeInt(codecorpusList.getCode(i));
        }
        int ficheSearchResultGroupCount = ficheSearchResult.getFicheSearchResultGroupCount();
        primitivesWriter.writeInt(ficheSearchResultGroupCount);
        for (int i = 0; i < ficheSearchResultGroupCount; i++) {
            FicheSearchResultGroup ficheSearchResultGroup = ficheSearchResult.getFicheSearchResultGroup(i);
            primitivesWriter.writeInt(ficheSearchResultGroup.getCategoryRank());
            int count = ficheSearchResultGroup.getFicheSearchResultInfoCount();
            primitivesWriter.writeInt(count);
            for (int j = 0; j < count; j++) {
                writeFicheSearchResultInfo(primitivesWriter, ficheSearchResultGroup.getFicheSearchResultInfo(j));
            }
        }
        for (int i = 0; i < motcleCount; i++) {
            writeMotcleSearchResultInfo(primitivesWriter, ficheSearchResult.getMotcleResult(i));
        }
    }

    public static FicheSearchResult read(File file) throws IOException {
        DataInputStream dataInput = new DataInputStream(new FileInputStream(file));
        DataInputPrimitivesReader primReader = new DataInputPrimitivesReader(dataInput);
        return FicheSearchResultStorage.read(primReader);
    }

    public static FicheSearchResult read(PrimitivesReader primitivesReader) throws IOException {
        int version = primitivesReader.readInt();
        if (version != CACHE_VERSION) {
            return null;
        }
        FicheSearchSource ficheSearchSource = readFicheSearchSource(primitivesReader);
        FicheSearchResultBuilder builder = new FicheSearchResultBuilder(ficheSearchSource, false);
        int ficheCount = primitivesReader.readInt();
        builder.setFicheCount(ficheCount);
        builder.setFicheMaximum(primitivesReader.readInt());
        int motcleCount = primitivesReader.readInt();
        builder.setMotcleCount(motcleCount);
        int corpusCount = primitivesReader.readInt();
        for (int i = 0; i < corpusCount; i++) {
            builder.addCodecorpus(primitivesReader.readInt());
        }
        int ficheSearchResultGroupCount = primitivesReader.readInt();
        if (ficheCount > 0) {
            for (int i = 0; i < ficheSearchResultGroupCount; i++) {
                FicheSearchResultGroupBuilder groupBuilder = new FicheSearchResultGroupBuilder();
                groupBuilder.setCategoryRank(primitivesReader.readInt());
                int count = primitivesReader.readInt();
                groupBuilder.ensureInfoCapacity(count);
                for (int j = 0; j < count; j++) {
                    groupBuilder.addFicheSearchResultInfo(readFicheSearchResultInfo(primitivesReader));
                }
                builder.addFicheSearchResultGroup(groupBuilder.toFicheSearchResultGroup());
            }
        }
        if (motcleCount > 0) {
            for (int i = 0; i < motcleCount; i++) {
                builder.addMotcleSearchResultInfo(readMotcleSearchResultInfo(primitivesReader));
            }
        }
        return builder.toFicheSearchResult();
    }

    private static void writeFicheSearchResultInfo(PrimitivesWriter primitivesWriter, FicheSearchResultInfo ficheSearchResultInfo) throws IOException {
        primitivesWriter.writeInt(ficheSearchResultInfo.getCode());
        int motcleCount = ficheSearchResultInfo.getMotcleCount();
        primitivesWriter.writeInt(motcleCount);
        for (int i = 0; i < motcleCount; i++) {
            primitivesWriter.writeInt(ficheSearchResultInfo.getMotcleCode(i));
        }
        int inAlineaOccurrenceListCount = ficheSearchResultInfo.getInAlineaOccurrenceListCount();
        primitivesWriter.writeInt(inAlineaOccurrenceListCount);
        for (int i = 0; i < inAlineaOccurrenceListCount; i++) {
            InAlineaOccurrenceList inAlineaOccurrenceList = ficheSearchResultInfo.getInAlineaOccurrenceList(i);
            primitivesWriter.writeInt(inAlineaOccurrenceList.getAlineaInteger());
            writeSearchTokenPositionList(primitivesWriter, inAlineaOccurrenceList);
        }
        int[] scoreArray = ficheSearchResultInfo.getScoreArray();
        int scoreLength = scoreArray.length;
        primitivesWriter.writeInt(scoreLength);
        for (int i = 0; i < scoreLength; i++) {
            primitivesWriter.writeInt(scoreArray[i]);
        }
    }

    private static void writeMotcleSearchResultInfo(PrimitivesWriter primitivesWriter, MotcleSearchResultInfo motcleSearchResultInfo) throws IOException {
        primitivesWriter.writeInt(motcleSearchResultInfo.getCode());
        int inLangOccurrenceListCount = motcleSearchResultInfo.getInLangOccurrenceListCount();
        primitivesWriter.writeInt(inLangOccurrenceListCount);
        for (int i = 0; i < inLangOccurrenceListCount; i++) {
            InLangOccurrenceList inLangOccurrenceList = motcleSearchResultInfo.getInLangOccurrenceList(i);
            primitivesWriter.writeString(inLangOccurrenceList.getLang().toString());
            writeSearchTokenPositionList(primitivesWriter, inLangOccurrenceList);
        }
    }

    private static void writeSearchTokenPositionList(PrimitivesWriter primitivesWriter, SearchTokenOccurrenceList searchTokenOccurrenceList) throws IOException {
        int size = searchTokenOccurrenceList.getSearchTokenOccurrenceCount();
        primitivesWriter.writeInt(size);
        for (int j = 0; j < size; j++) {
            SearchTokenOccurrence tokenOccurrence = searchTokenOccurrenceList.getSearchTokenOccurrence(j);
            primitivesWriter.writeInt(tokenOccurrence.getOperandNumber());
            primitivesWriter.writeInt(tokenOccurrence.getBeginIndex());
            primitivesWriter.writeInt(tokenOccurrence.getLength());
        }
    }

    private static FicheSearchResultInfo readFicheSearchResultInfo(PrimitivesReader primitivesReader) throws IOException {
        int code = primitivesReader.readInt();
        int motcleCount = primitivesReader.readInt();
        int[] motcleCodeArray = null;
        if (motcleCount > 0) {
            motcleCodeArray = new int[motcleCount];
            for (int i = 0; i < motcleCount; i++) {
                motcleCodeArray[i] = primitivesReader.readInt();
            }
        }
        int inAlineaOccurrenceListCount = primitivesReader.readInt();
        InAlineaOccurrenceList[] inAlineaOccurrenceListArray = null;
        if (inAlineaOccurrenceListCount > 0) {
            inAlineaOccurrenceListArray = new InAlineaOccurrenceList[inAlineaOccurrenceListCount];
            for (int i = 0; i < inAlineaOccurrenceListCount; i++) {
                int alineaInteger = primitivesReader.readInt();
                SearchTokenOccurrence[] searchTokenOccurrenceArray = readSearchTokenPositionArray(primitivesReader);
                inAlineaOccurrenceListArray[i] = ResultUtils.toInAlineaOccurrenceList(alineaInteger, searchTokenOccurrenceArray);
            }
        }
        int scoreLength = primitivesReader.readInt();
        int[] scoreArray = new int[scoreLength];
        for (int i = 0; i < scoreLength; i++) {
            scoreArray[i] = primitivesReader.readInt();
        }
        FicheSearchResultInfoBuilder ficheSearchResultInfoBuilder = new FicheSearchResultInfoBuilder();
        ficheSearchResultInfoBuilder.setCode(code);
        ficheSearchResultInfoBuilder.setMotcleCodeArray(motcleCodeArray);
        ficheSearchResultInfoBuilder.setInAlineaOccurrenceListArray(inAlineaOccurrenceListArray);
        ficheSearchResultInfoBuilder.setScoreArray(scoreArray);
        return ficheSearchResultInfoBuilder.toFicheSearchResultInfo();
    }

    private static MotcleSearchResultInfo readMotcleSearchResultInfo(PrimitivesReader primitivesReader) throws IOException {
        int code = primitivesReader.readInt();
        int inLangOccurrenceListCount = primitivesReader.readInt();
        InLangOccurrenceList[] inLangOccurrenceListArray = null;
        if (inLangOccurrenceListCount > 0) {
            inLangOccurrenceListArray = new InLangOccurrenceList[inLangOccurrenceListCount];
            for (int i = 0; i < inLangOccurrenceListCount; i++) {
                Lang lang = Lang.build(primitivesReader.readString());
                SearchTokenOccurrence[] searchTokenOccurrenceArray = readSearchTokenPositionArray(primitivesReader);
                inLangOccurrenceListArray[i] = ResultUtils.toInLangOccurrenceList(lang, searchTokenOccurrenceArray);
            }
        }
        MotcleSearchResultBuilder motcleSearchResultInfoBuilder = new MotcleSearchResultBuilder();
        motcleSearchResultInfoBuilder.setCode(code);
        motcleSearchResultInfoBuilder.setInLangOccurrenceListArray(inLangOccurrenceListArray);
        return motcleSearchResultInfoBuilder.toMotcleSearchResultInfo();
    }

    private static SearchTokenOccurrence[] readSearchTokenPositionArray(PrimitivesReader primitivesReader) throws IOException {
        int searchTokenPositionCount = primitivesReader.readInt();
        if (searchTokenPositionCount == 0) {
            return null;
        }
        SearchTokenOccurrence[] result = new SearchTokenOccurrence[searchTokenPositionCount];
        for (int i = 0; i < searchTokenPositionCount; i++) {
            int operandNumber = primitivesReader.readInt();
            int beginIndex = primitivesReader.readInt();
            int length = primitivesReader.readInt();
            result[i] = SearchTokenOccurrenceBuilder.toSearchTokenOccurrence(operandNumber, beginIndex, length);
        }
        return result;
    }

    private static void writeFicheSearchSource(PrimitivesWriter primitivesWriter, FicheSearchSource ficheSearchSource) throws IOException {
        SearchOptionsDef searchOptionsDef = ficheSearchSource.getSearchOptionsDef();
        QId qId = ficheSearchSource.getQId();
        String qIdString;
        if (qId != null) {
            qIdString = qId.toString();
        } else {
            qIdString = "";
        }
        primitivesWriter.writeString(qIdString);
        primitivesWriter.writeString(ficheSearchSource.getCanonicalQ().toString());
        Lang langUi = searchOptionsDef.getLangUi();
        if (langUi != null) {
            primitivesWriter.writeString(langUi.toString());
        } else {
            primitivesWriter.writeString("");
        }
        LangList langList = searchOptionsDef.getFilterLangList();
        int langCount = langList.getLangCount();
        primitivesWriter.writeInt(langCount);
        for (int i = 0; i < langCount; i++) {
            primitivesWriter.writeString(langList.getLang(i).toString());
        }
    }

    private static FicheSearchSource readFicheSearchSource(PrimitivesReader primitivesReader) throws IOException {
        String qIdString = primitivesReader.readString();
        QId qId = null;
        if (qIdString.length() > 0) {
            try {
                qId = QId.parse(qIdString);
            } catch (ParseException pe) {
            }
        }
        String q = primitivesReader.readString();
        CanonicalQ canonicalQ = CanonicalQ.newInstance(q);
        SearchOptionsDefBuilder searchOptionsDefBuilder = new SearchOptionsDefBuilder();
        String langString = primitivesReader.readString();
        if (!langString.isEmpty()) {
            searchOptionsDefBuilder.setLangUi(Lang.build(langString));
        }
        int langCount = primitivesReader.readInt();
        for (int i = 0; i < langCount; i++) {
            searchOptionsDefBuilder.addFilterLang(Lang.build(primitivesReader.readString()));
        }
        return FicheSearchSourceBuilder.build(qId, canonicalQ, searchOptionsDefBuilder.toSearchOptionsDef());
    }

}
