/* ScrutariLib_Xml - Copyright (c) 2014-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.xml;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashSet;
import java.util.Set;
import net.mapeadores.util.exceptions.NestedLibraryException;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.primitives.DuoKey;
import net.scrutari.data.IndexationData;
import net.scrutari.datauri.BaseURI;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.FicheURI;
import net.scrutari.datauri.MotcleURI;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.ThesaurusURI;
import net.scrutari.datauri.URIParseException;
import net.scrutari.datauri.URIParser;
import net.scrutari.db.api.DataCoder;
import net.scrutari.db.api.caches.DataCacheWriter;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlPullParserFactory;


/**
 *
 * @author Vincent Calame
 */
public class IndexationXmlEngine {

    private final static short WAITING_FOR_GROUP = 1;
    private final static short ON_GROUP = 2;
    private final XmlPullParserFactory factory;
    private final DataCacheWriter dataCacheWriter;
    private final DataCoder dataCoder;
    private final MessageHandler messageHandler;
    private final Set<DuoKey> indexationKeySet = new HashSet<DuoKey>();

    public IndexationXmlEngine(DataCacheWriter dataCacheWriter, DataCoder dataCoder, MessageHandler messageHandler) {
        this.dataCacheWriter = dataCacheWriter;
        this.dataCoder = dataCoder;
        this.messageHandler = messageHandler;
        try {
            this.factory = XmlPullParserFactory.newInstance(System.getProperty(XmlPullParserFactory.PROPERTY_NAME), null);
        } catch (XmlPullParserException xppe) {
            throw new NestedLibraryException(xppe);
        }
    }

    public void run(InputStream inputStream, BaseURI baseURI) throws XmlPullParserException, IOException {
        XmlPullParser xpp = factory.newPullParser();
        xpp.setInput(inputStream, null);
        int currentLevel = 0;
        short state = WAITING_FOR_GROUP;
        int eventType = xpp.getEventType();
        int currentCorpus = -1;
        int currentThesaurus = -1;
        CorpusURI corpusURI = null;
        ThesaurusURI thesaurusURI = null;
        String xpath = "";
        do {
            if (eventType == XmlPullParser.START_TAG) {
                currentLevel++;
                String tagName = xpp.getName();
                switch (state) {
                    case WAITING_FOR_GROUP:
                        if ((currentLevel == 2) && isIndexationGroupElement(tagName)) {
                            xpath = "/base/" + tagName;
                            String corpuspath = null;
                            String thesauruspath = null;
                            int count = xpp.getAttributeCount();
                            for (int i = 0; i < count; i++) {
                                String attributeName = xpp.getAttributeName(i);
                                if (attributeName.equals("corpus-path")) {
                                    corpuspath = xpp.getAttributeValue(i);
                                } else if (attributeName.equals("thesaurus-path")) {
                                    thesauruspath = xpp.getAttributeValue(i);
                                }
                            }
                            boolean error = false;
                            if (corpuspath == null) {
                                addWarning("_ error.empty.xml.attribute", LogUtils.toValue(xpath, "corpus-path"));
                                error = true;
                            }
                            if (thesauruspath == null) {
                                addWarning("_ error.empty.xml.attribute", LogUtils.toValue(xpath, "thesaurus-path"));
                                error = true;
                            }
                            if (!error) {
                                xpath = xpath + "[@corpus-path='" + corpuspath + "' and @thesaurus-path='" + thesauruspath + "']";
                                try {
                                    ScrutariDataURI scrutariDataURI = URIParser.parse(corpuspath);
                                    if (scrutariDataURI instanceof CorpusURI) {
                                        corpusURI = (CorpusURI) scrutariDataURI;
                                    } else {
                                        addWarning("_ error.wrong.xml.attributevalue", LogUtils.toValue(xpath, "corpus-path", corpuspath));
                                        error = true;
                                    }
                                } catch (URIParseException uriParseException) {
                                    corpusURI = new CorpusURI(baseURI, corpuspath);
                                }
                                try {
                                    ScrutariDataURI scrutariDataURI = URIParser.parse(thesauruspath);
                                    if (scrutariDataURI instanceof ThesaurusURI) {
                                        thesaurusURI = (ThesaurusURI) scrutariDataURI;
                                    } else {
                                        addWarning("_ error.wrong.xml.attributevalue", LogUtils.toValue(xpath, "thesaurus-path", thesauruspath));
                                        error = true;
                                    }
                                } catch (URIParseException uriParseException) {
                                    thesaurusURI = new ThesaurusURI(baseURI, thesauruspath);
                                }
                                if (!error) {
                                    currentCorpus = dataCoder.checkCode(corpusURI);
                                    if (currentCorpus == -1) {
                                        addWarning("_ error.unknown.xml.attributevalue", LogUtils.toValue(xpath, "corpus-path", corpuspath));
                                        error = true;
                                    }
                                    currentThesaurus = dataCoder.checkCode(thesaurusURI);
                                    if (currentThesaurus == -1) {
                                        addWarning("_ error.unknown.xml.attributevalue", LogUtils.toValue(xpath, "thesaurus-path", thesauruspath));
                                        error = true;
                                    }
                                    if (!error) {
                                        state = ON_GROUP;
                                    }
                                }
                            }
                        }
                        break;
                    case ON_GROUP:
                        if ((currentLevel == 3) && isIndexationElement(tagName)) {
                            boolean error = false;
                            String ficheid = null;
                            String motcleid = null;
                            String poidsString = null;
                            int count = xpp.getAttributeCount();
                            for (int i = 0; i < count; i++) {
                                String attributeName = xpp.getAttributeName(i);
                                if (attributeName.equals("fiche-id")) {
                                    ficheid = xpp.getAttributeValue(i);
                                } else if (attributeName.equals("motcle-id")) {
                                    motcleid = xpp.getAttributeValue(i);
                                } else if (attributeName.equals("poids")) {
                                    poidsString = xpp.getAttributeValue(i);
                                } else if (attributeName.equals("weight")) {
                                    poidsString = xpp.getAttributeValue(i);
                                }
                            }
                            if (ficheid == null) {
                                addWarning("_ error.empty.xml.attribute", LogUtils.toValue(xpath + "/" + tagName, "fiche-id"));
                                error = true;
                            }
                            if (motcleid == null) {
                                addWarning("_ error.empty.xml.attribute", LogUtils.toValue(xpath + "/" + tagName, "motcle-id"));
                                error = true;
                            }
                            if (!error) {
                                String indexationxpath = xpath + "/" + tagName + "[@fiche-id='" + ficheid + "' @motcle-id='" + motcleid + "']";
                                FicheURI ficheURI = new FicheURI(corpusURI, ficheid);
                                MotcleURI motcleURI = new MotcleURI(thesaurusURI, motcleid);
                                int codefiche = dataCoder.checkCode(ficheURI);
                                int codemotcle = dataCoder.checkCode(motcleURI);
                                if (codefiche == -1) {
                                    addWarning("_ error.unknown.xml.attributevalue", LogUtils.toValue(indexationxpath, "fiche-id", ficheid));
                                    error = true;
                                }
                                if (codemotcle == -1) {
                                    addWarning("_ error.unknown.xml.attributevalue", LogUtils.toValue(indexationxpath, "motcle-id", motcleid));
                                    error = true;
                                }
                                if (!error) {
                                    DuoKey indexationKey = new DuoKey(codefiche, codemotcle);
                                    if (indexationKeySet.contains(indexationKey)) {
                                        addWarning("_ error.existing.xml.attributevalue", LogUtils.toValue(indexationxpath, "fiche-id|motcle-id", ficheid + "|" + motcleid));
                                        error = true;
                                    }
                                    if (!error) {
                                        indexationKeySet.add(indexationKey);
                                        int poids = 1;
                                        if (poidsString != null) {
                                            try {
                                                poids = Integer.parseInt(poidsString);
                                                if (poids < 1) {
                                                    poids = 1;
                                                }
                                            } catch (NumberFormatException nfe) {
                                            }
                                        }
                                        dataCacheWriter.cacheIndexationData(new IndexationData(codefiche, currentCorpus, codemotcle, currentThesaurus, poids));
                                    }
                                }
                            }
                        }
                        break;
                }
            } else if (eventType == XmlPullParser.END_TAG) {
                String tagName = xpp.getName();
                if ((currentLevel == 2) && (state == ON_GROUP) && isIndexationGroupElement(tagName)) {
                    state = WAITING_FOR_GROUP;
                }
                currentLevel--;
            }
            eventType = xpp.next();
        } while (eventType != XmlPullParser.END_DOCUMENT);

    }

    private void addWarning(String key, Object value) {
        messageHandler.addMessage(ScrutariXmlConstants.WARNING_CATEGORY, LocalisationUtils.toMessage(key, value));
    }

    private static boolean isIndexationGroupElement(String tagName) {
        return (tagName.equals("indexation-group"));
    }

    private static boolean isIndexationElement(String tagName) {
        return (tagName.equals("indexation"));
    }

    public static boolean isElement(String tagName) {
        return (isIndexationGroupElement(tagName)) || (isIndexationElement(tagName));
    }

}
