/* SctEngine - Copyright (c) 2010-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.json;

import fr.exemole.sctserver.api.EngineMetadata;
import java.net.URL;
import java.util.HashSet;
import java.util.Set;
import net.mapeadores.util.attr.AttributeDef;
import net.mapeadores.util.attr.AttributeDefList;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.json.CommonJson;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelHolder;
import net.mapeadores.util.text.LabelUtils;
import net.scrutari.data.ScrutariAttributes;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.db.api.stats.CountStats;
import net.scrutari.db.api.stats.FicheStats;
import net.scrutari.searchengine.api.result.InLangOccurrenceList;
import net.scrutari.searchengine.api.result.SearchTokenOccurrence;
import net.scrutari.searchengine.api.result.SearchTokenOccurrenceList;


/**
 *
 * @author Vincent Calame
 */
public final class JsonUtils {

    private JsonUtils() {

    }

    public static void writeMLib(JSONWriter jsonWriter, String lib, SearchTokenOccurrenceList positionList) throws JSONException {
        if (positionList == null) {
            jsonWriter.array();
            jsonWriter.value(lib);
            jsonWriter.endArray();
            return;
        }
        PositionListBuilder builder = new PositionListBuilder(jsonWriter, lib, positionList);
        builder.buildArray();
    }

    public static void addFicheStats(JSONWriter jsonWriter, FicheStats ficheStats) throws JSONException {
        jsonWriter.key("langArray");
        jsonWriter.array();
        int count = ficheStats.getLangCount();
        for (int i = 0; i < count; i++) {
            jsonWriter.object();
            jsonWriter.key("lang");
            jsonWriter.value(ficheStats.getLang(i).toString());
            jsonWriter.key("fiche");
            jsonWriter.value(ficheStats.getFicheCount(i));
            jsonWriter.endObject();
        }
        jsonWriter.endArray();
    }

    public static void addCountStats(JSONWriter jsonWriter, CountStats stats) throws JSONException {
        int baseCount = stats.getBaseCount();
        if (baseCount > - 1) {
            jsonWriter.key("base");
            jsonWriter.value(baseCount);
        }
        int corpusCount = stats.getCorpusCount();
        if (corpusCount > - 1) {
            jsonWriter.key("corpus");
            jsonWriter.value(corpusCount);
        }
        int thesaurusCount = stats.getThesaurusCount();
        if (thesaurusCount > -1) {
            jsonWriter.key("thesaurus");
            jsonWriter.value(thesaurusCount);
        }
        int ficheCount = stats.getFicheCount();
        if (ficheCount > -1) {
            jsonWriter.key("fiche");
            jsonWriter.value(ficheCount);
        }
        int motcleCount = stats.getMotcleCount();
        if (motcleCount > -1) {
            jsonWriter.key("motcle");
            jsonWriter.value(motcleCount);
        }
        int indexationCount = stats.getIndexationCount();
        if (indexationCount > -1) {
            jsonWriter.key("indexation");
            jsonWriter.value(indexationCount);
        }
        int lexieCount = stats.getLexieCount();
        if (lexieCount > -1) {
            jsonWriter.key("lexie");
            jsonWriter.value(lexieCount);
        }
    }

    public static void addAttributes(JSONWriter jsonWriter, AttributeKeyCoder attributeKeyCoder, Lang langUi, int version) throws JSONException {
        jsonWriter.key("attributes");
        jsonWriter.object();
        addAttributeDefList(jsonWriter, attributeKeyCoder, AttributeKeyCoder.PRIMARY_GROUP, langUi, version);
        addAttributeDefList(jsonWriter, attributeKeyCoder, AttributeKeyCoder.SECONDARY_GROUP, langUi, version);
        addAttributeDefList(jsonWriter, attributeKeyCoder, AttributeKeyCoder.TECHNICAL_GROUP, langUi, version);
        jsonWriter.endObject();
    }

    public static void addMetadataProperties(JSONWriter jsonWriter, String engineName, EngineMetadata engineMetadata, Lang lang, boolean withLabelMap) throws JSONException {
        CommonJson.defaultLabel(jsonWriter, engineMetadata.getTitle(), lang, "title", engineName);
        if (withLabelMap) {
            jsonWriter.key("titleLabelMap");
            CommonJson.object(jsonWriter, (LabelHolder) engineMetadata.getTitle());
        }
        CommonJson.defaultLabel(jsonWriter, engineMetadata.getDescription(), lang, "description", null);
        if (withLabelMap) {
            jsonWriter.key("descriptionLabelMap");
            CommonJson.object(jsonWriter, (LabelHolder) engineMetadata.getDescription());
        }
        printUrl(jsonWriter, "icon", engineMetadata.getIcon());
        printUrl(jsonWriter, "website", engineMetadata.getWebsite());
        jsonWriter.key("attrMap");
        CommonJson.object(jsonWriter, (AttributeHolder) engineMetadata);
    }


    private static void addAttributeDefList(JSONWriter jsonWriter, AttributeKeyCoder attributeKeyCoder, String name, Lang langUi, int version) throws JSONException {
        AttributeDefList attributeDefList = attributeKeyCoder.getAttributeDefList(name);
        int count = attributeDefList.getAttributeDefCount();
        jsonWriter.key(name);
        jsonWriter.array();
        for (int i = 0; i < count; i++) {
            AttributeDef attributeDef = attributeDefList.getAttributeDef(i);
            jsonWriter.object();
            AttributeKey attributeKey = attributeDef.getAttributeKey();
            if (version == 0) {
                jsonWriter.key("key");
            } else {
                jsonWriter.key("name");
            }
            jsonWriter.value(attributeKey.toString());
            if (version > 0) {
                jsonWriter.key("ns");
                jsonWriter.value(attributeKey.getNameSpace());
                jsonWriter.key("key");
                jsonWriter.value(attributeKey.getLocalKey());
            }
            String type = attributeDef.getFormatType();
            if (type.isEmpty()) {
                type = ScrutariAttributes.INLINE_TYPE;
            }
            jsonWriter.key("type");
            jsonWriter.value(type);
            LabelHolder labelHolder = attributeDef.getLabelHolder();
            Label label = LabelUtils.getLangPartCheckedLabel(labelHolder, langUi);
            if (label == null) {
                label = LabelUtils.getFirstLabel(labelHolder);
            }
            if (version == 0) {
                jsonWriter.key("lib");
            } else {
                jsonWriter.key("title");
            }
            if (label != null) {
                jsonWriter.value(label.getLabelString());
            } else {
                jsonWriter.value(attributeDef.getAttributeKey().toString());
            }
            jsonWriter.endObject();
        }
        jsonWriter.endArray();
    }


    public static Set<Integer> getOperandSet(InLangOccurrenceList occurrenceList) {
        Set<Integer> set = new HashSet<Integer>();
        int count = occurrenceList.getSearchTokenOccurrenceCount();
        for (int i = 0; i < count; i++) {
            SearchTokenOccurrence searchTokenOccurrence = occurrenceList.getSearchTokenOccurrence(i);
            set.add(searchTokenOccurrence.getOperandNumber());
        }
        return set;
    }

    public static boolean isCovered(InLangOccurrenceList occurrenceList, Set<Integer> set) {
        int count = occurrenceList.getSearchTokenOccurrenceCount();
        for (int i = 0; i < count; i++) {
            SearchTokenOccurrence searchTokenOccurrence = occurrenceList.getSearchTokenOccurrence(i);
            if (!set.contains(searchTokenOccurrence.getOperandNumber())) {
                return false;
            }
        }
        return true;
    }

    private static void printUrl(JSONWriter jsonWriter, String key, URL url) throws JSONException {
        if (url == null) {
            return;
        }
        jsonWriter.key(key);
        jsonWriter.value(url.toString());
    }


    private static class PositionListBuilder {

        private final JSONWriter jsonWriter;
        SearchTokenOccurrenceList positionList;
        StringBuilder buf;
        String s;
        int openIndex;
        int closeIndex;
        int currentPosition;

        /*
         * Constructeur unique.
         */
        private PositionListBuilder(JSONWriter jsonWriter, String s, SearchTokenOccurrenceList positionList) {
            this.jsonWriter = jsonWriter;
            this.positionList = positionList;
            this.s = s;
            currentPosition = 0;
            SearchTokenOccurrence firstTokenOccurrence = positionList.getSearchTokenOccurrence(0);
            openIndex = firstTokenOccurrence.getBeginIndex();
            closeIndex = openIndex + firstTokenOccurrence.getLength() - 1;
            buf = new StringBuilder();
        }

        /*
         * Lancement de l'écriture.
         */
        private void buildArray() throws JSONException {
            jsonWriter.array();
            int length = s.length();
            boolean onObject = false;
            for (int i = 0; i < length; i++) {
                if (i == openIndex) {
                    flushBuffer(false);
                    jsonWriter.object();
                    jsonWriter.key("i");
                    jsonWriter.value(positionList.getSearchTokenOccurrence(currentPosition).getOperandNumber());
                    jsonWriter.key("s");
                    onObject = true;
                }
                char carac = s.charAt(i);
                buf.append(carac);
                if (i == closeIndex) {
                    flushBuffer(true);
                    jsonWriter.endObject();
                    increment();
                    onObject = false;
                }
            }
            flushBuffer(false);
            if (onObject) {
                jsonWriter.endObject();
            }
            jsonWriter.endArray();
        }

        void flushBuffer(boolean withEmpty) throws JSONException {
            String bufString = buf.toString();
            if ((!withEmpty) && (bufString.length() == 0)) {
                return;
            }
            jsonWriter.value(bufString);
            buf = new StringBuilder();
        }

        /*
         * gère les éventuels recouvrements de position. Cela peut aller jusqu'à
         * générer un élément s vide (<s token-index=""/>) si une position est
         * complètement incluse dans une autre.
         */
        void increment() throws JSONException {
            if (currentPosition == (positionList.getSearchTokenOccurrenceCount() - 1)) {
                openIndex = -1;
                closeIndex = -1;
                return;
            }
            currentPosition++;
            SearchTokenOccurrence tokenOccurrence = positionList.getSearchTokenOccurrence(currentPosition);
            int beginIndex = tokenOccurrence.getBeginIndex();
            int endIndex = beginIndex + tokenOccurrence.getLength() - 1;
            if (beginIndex > closeIndex) {
                openIndex = beginIndex;
                closeIndex = endIndex;
                return;
            }
            openIndex = -1;
            jsonWriter.object();
            jsonWriter.key("i");
            jsonWriter.value(tokenOccurrence.getOperandNumber());
            if (endIndex <= closeIndex) {
                jsonWriter.endObject();
                increment();
            } else {
                jsonWriter.key("s");
                closeIndex = endIndex;
            }
        }

    }

}
