/* SctEngine - Copyright (c) 2014-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.SctEngine;
import fr.exemole.sctserver.json.BaseDataJson;
import fr.exemole.sctserver.json.CategoryJson;
import fr.exemole.sctserver.json.CorpusDataJson;
import fr.exemole.sctserver.json.JsonUtils;
import fr.exemole.sctserver.json.ThesaurusDataJson;
import fr.exemole.sctserver.request.DataFilter;
import fr.exemole.sctserver.tools.EngineUtils;
import java.io.IOException;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.json.JSONException;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.MessageLocalisation;
import net.mapeadores.util.text.StringUtils;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.db.api.stats.FicheStats;
import net.scrutari.db.api.stats.ScrutariDBStats;
import net.scrutari.searchengine.api.options.Category;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;


/**
 *
 * @author Vincent Calame
 */
public class EngineInfoJsonProducer extends AbstractJsonProducer {

    private final int version;
    private final ScrutariSession scrutariSession;
    private final Lang langUi;
    private final DataFilter dataFilter;

    public EngineInfoJsonProducer(int version, ScrutariSession scrutariSession, Lang langUi, DataFilter dataFilter) {
        if (langUi == null) {
            throw new IllegalArgumentException("langUi is null");
        }
        this.version = version;
        this.scrutariSession = scrutariSession;
        this.langUi = langUi;
        this.dataFilter = dataFilter;
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        ScrutariDB scrutariDB = scrutariSession.getScrutariDB();
        SctEngine engine = scrutariSession.getEngine();
        DataAccess dataAccess = scrutariDB.openDataAccess();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        FicheStats engineFicheStats = scrutariDB.getStats().getEngineFicheStats();
        boolean withStats = dataFilter.isWithStats();
        ScrutariDBStats stats;
        if (dataFilter.isWithStats()) {
            stats = scrutariDB.getStats();
        } else {
            stats = null;
        }
        GlobalSearchOptions globalSearchOptions = scrutariSession.getGlobalSearchOptions();
        MessageLocalisation messageLocalisation = EngineUtils.getMessageLocalisation(scrutariSession.getEngine(), langUi);
        try {
            jsonWriter.object();
            jsonWriter.key("engineInfo");
            jsonWriter.object();
            if (dataFilter.isWithBase()) {
                jsonWriter.key("baseMap");
                jsonWriter.object();
                int baseCount = dataAccess.getBaseCount();
                for (int i = 0; i < baseCount; i++) {
                    BaseData baseData = dataAccess.getBaseDataByIndex(i);
                    jsonWriter.key("code_" + baseData.getCodeBase());
                    BaseDataJson.object(jsonWriter, baseData, langUi, stats);
                }
                jsonWriter.endObject();
            }
            if (dataFilter.isWithCorpus()) {
                jsonWriter.key("corpusMap");
                jsonWriter.object();
                int count = dataAccess.getCorpusCount();
                for (int i = 0; i < count; i++) {
                    CorpusData corpusData = dataAccess.getCorpusDataByIndex(i);
                    jsonWriter.key("code_" + corpusData.getCodeCorpus());
                    CorpusDataJson.object(jsonWriter, corpusData, langUi, stats);
                }
                jsonWriter.endObject();
            }
            if (dataFilter.isWithThesaurus()) {
                jsonWriter.key("thesaurusMap");
                jsonWriter.object();
                int count = dataAccess.getThesaurusCount();
                for (int i = 0; i < count; i++) {
                    ThesaurusData thesaurusData = dataAccess.getThesaurusDataByIndex(i);
                    jsonWriter.key("code_" + thesaurusData.getCodeThesaurus());
                    ThesaurusDataJson.object(jsonWriter, thesaurusData, langUi, stats);
                }
                jsonWriter.endObject();
            }
            if (withStats) {
                jsonWriter.key("stats");
                jsonWriter.object();
                JsonUtils.addCountStats(jsonWriter, stats.getEngineCountStats());
                JsonUtils.addFicheStats(jsonWriter, engineFicheStats);
                jsonWriter.endObject();
            }
            if (dataFilter.isWithCategory()) {
                int categoryCount = globalSearchOptions.getCategoryCount();
                if (categoryCount > 0) {
                    jsonWriter.key("categoryMap");
                    jsonWriter.object();
                    for (int i = 0; i < categoryCount; i++) {
                        Category category = globalSearchOptions.getCategoryByIndex(i);
                        jsonWriter.key(category.getCategoryDef().getName());
                        CategoryJson.object(jsonWriter, category, langUi, withStats);
                    }
                    jsonWriter.endObject();
                }
            }
            if (dataFilter.isWithLang()) {
                int count = engineFicheStats.getLangCount();
                jsonWriter.key("langMap");
                jsonWriter.object();
                for (int i = 0; i < count; i++) {
                    Lang lang = engineFicheStats.getLang(i);
                    String code = lang.toString();
                    jsonWriter.key(code);
                    String value = messageLocalisation.toString(code, null);
                    if (value != null) {
                        value = StringUtils.getFirstPart(value);
                    } else {
                        value = code;
                    }
                    jsonWriter.value(value);
                }
                jsonWriter.endObject();
            }
            if (dataFilter.isWithAttributes()) {
                JsonUtils.addAttributes(jsonWriter, scrutariDB.getAttributeKeyCoder(), langUi, version);
            }
            if (dataFilter.isWithMetadata()) {
                jsonWriter.key("metadata");
                jsonWriter.object();
                JsonUtils.addMetadataProperties(jsonWriter, engine.getEngineName(), engine.getEngineMetadata(), langUi, false);
                jsonWriter.endObject();
            }
            jsonWriter.endObject();
            writeWarnings(jsonWriter);
            jsonWriter.endObject();
        } catch (JSONException je) {
            Throwable throwable = je.getCause();
            if (throwable == null) {
                throw new ShouldNotOccurException(je.getMessage(), je);
            } else {
                if (throwable instanceof IOException) {
                    throw ((IOException) throwable);
                } else {
                    throw new ShouldNotOccurException(throwable.getMessage(), (Exception) throwable);
                }
            }
        }
    }

}
