/* SctServer - Copyright (c) 2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.dom;

import fr.exemole.sctserver.api.EngineMetadata;
import fr.exemole.sctserver.tools.ErrorUtils;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.ParseException;
import net.mapeadores.util.attr.AppendableAttributeHolderBuilder;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.text.ChangeLabelHolderBuilder;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelHolder;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class MetadataDOMReader {

    private final ChangeLabelHolderBuilder titleLabelHolderBuilder = new ChangeLabelHolderBuilder();
    private final ChangeLabelHolderBuilder descriptionLabelHolderBuilder = new ChangeLabelHolderBuilder();
    private final AppendableAttributeHolderBuilder attributeHolderBuilder = new AppendableAttributeHolderBuilder();
    private final MessageHandler messageHandler;
    private URL iconUrl;
    private URL websiteUrl;
    private Lang langUi;

    public MetadataDOMReader(MessageHandler messageHandler) {
        this.messageHandler = messageHandler;
    }

    public EngineMetadata readMetadata(Element element, String xpath) {
        RootHandler rootHandler = new RootHandler(xpath + "/" + element.getTagName());
        DOMUtils.readChildren(element, rootHandler);
        return new InternalEngineMetadata(titleLabelHolderBuilder.toLabelHolder(), descriptionLabelHolderBuilder.toLabelHolder(), iconUrl, websiteUrl, langUi, attributeHolderBuilder.toAttributeHolder());
    }


    private class RootHandler implements ElementHandler {

        private final String xpath;

        private RootHandler(String xpath) {
            this.xpath = xpath;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("title")) {
                LabelHolderHandler handler = new LabelHolderHandler(titleLabelHolderBuilder, xpath + "/" + tagName);
                DOMUtils.readChildren(element, handler);
            } else if (tagName.equals("description")) {
                LabelHolderHandler handler = new LabelHolderHandler(descriptionLabelHolderBuilder, xpath + "/" + tagName);
                DOMUtils.readChildren(element, handler);
            } else if (tagName.equals("icon")) {
                String data = XMLUtils.getData(element);
                if (data.length() > 0) {
                    iconUrl = getURL(data, xpath + "/" + tagName);
                }
            } else if (tagName.equals("website")) {
                String data = XMLUtils.getData(element);
                if (data.length() > 0) {
                    websiteUrl = getURL(data, xpath + "/" + tagName);
                }
            } else if (tagName.equals("default-lang-ui")) {
                String data = XMLUtils.getData(element);
                if (data.length() > 0) {
                    try {
                        langUi = Lang.parse(data);
                    } catch (ParseException pe) {
                        ErrorUtils.severe(messageHandler, "_ error.wrong.xml.tagvalue", xpath + tagName, data);
                    }
                }
            } else if (tagName.equals("attr")) {
                AttributeUtils.readAttrElement(attributeHolderBuilder, element);
            }
        }

        private URL getURL(String data, String urlXPath) {
            try {
                URL url = new URL(data);
                return url;
            } catch (MalformedURLException mue) {
                ErrorUtils.severe(messageHandler, "_ error.wrong.xml.tagvalue_url", urlXPath, data, mue.getMessage());
                return null;
            }
        }

    }


    private class LabelHolderHandler implements ElementHandler {

        private final ChangeLabelHolderBuilder labelHolderBuilder;
        private final String xpath;


        private LabelHolderHandler(ChangeLabelHolderBuilder labelHolderBuilder, String xpath) {
            this.labelHolderBuilder = labelHolderBuilder;
            this.xpath = xpath;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            if ((tagName.equals("label")) || (tagName.equals("lib"))) {
                try {
                    Label libelle = LabelUtils.readLabel(element);
                    if (libelle != null) {
                        labelHolderBuilder.putLabel(libelle);
                    }
                } catch (ParseException pe) {
                    ErrorUtils.severe(messageHandler, "_ error.wrong.xml.langattributevalue", xpath + tagName, element.getAttribute("xml:lang"));
                }
            }
        }

    }


    private static class InternalEngineMetadata implements EngineMetadata {

        private final LabelHolder titleLabelHolder;
        private final LabelHolder descriptionLabelHolder;
        private final URL iconUrl;
        private final URL websiteUrl;
        private final Lang langUi;
        private final AttributeHolder attributeHolder;

        private InternalEngineMetadata(LabelHolder titleLabelHolder, LabelHolder descriptionLabelHolder, URL iconUrl, URL websiteUrl, Lang langUi, AttributeHolder attributeHolder) {
            this.titleLabelHolder = titleLabelHolder;
            this.descriptionLabelHolder = descriptionLabelHolder;
            this.iconUrl = iconUrl;
            this.websiteUrl = websiteUrl;
            this.langUi = langUi;
            this.attributeHolder = attributeHolder;
        }

        @Override
        public LabelHolder getTitle() {
            return titleLabelHolder;
        }

        @Override
        public LabelHolder getDescription() {
            return descriptionLabelHolder;
        }

        @Override
        public URL getIcon() {
            return iconUrl;
        }

        @Override
        public URL getWebsite() {
            return websiteUrl;
        }

        @Override
        public Lang getDefaultLangUi() {
            return langUi;
        }

        @Override
        public Attribute getAttributeByKey(AttributeKey key) {
            return attributeHolder.getAttributeByKey(key);
        }

        @Override
        public int getAttributeCount() {
            return attributeHolder.getAttributeCount();
        }

        @Override
        public Attribute getAttributeByIndex(int i) {
            return attributeHolder.getAttributeByIndex(i);
        }

    }

}
