/* SctServer - Copyright (c) 2009-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.dom;

import fr.exemole.sctserver.api.collect.ScrutariSourceDef;
import fr.exemole.sctserver.tools.ErrorUtils;
import fr.exemole.sctserver.tools.collect.ScrutariSourceDefBuilder;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.XMLUtils;
import net.scrutari.datauri.BaseURI;
import net.scrutari.datauri.URIParseException;
import net.scrutari.datauri.URIParser;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class SourcesDOMReader {

    private final MessageHandler messageHandler;
    private final Map<String, ScrutariSourceDef> scrutariSourceDefMap = new LinkedHashMap<String, ScrutariSourceDef>();
    private final Set<BaseURI> baseURISet = new HashSet<BaseURI>();

    public SourcesDOMReader(MessageHandler messageHandler) {
        this.messageHandler = messageHandler;
    }

    public ScrutariSourceDef[] readSources(Element element_xml, String xpath) {
        xpath = xpath + "/" + element_xml.getTagName();
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("source")) {
                    addSource(element, xpath);
                } else {
                    ErrorUtils.warning(messageHandler, "_ error.unknown.xml.tag", xpath + "/" + tagName);
                }
            }
        }
        return scrutariSourceDefMap.values().toArray(new ScrutariSourceDef[scrutariSourceDefMap.size()]);
    }

    private void addSource(Element element_xml, String xpath) {
        xpath = xpath + "/" + element_xml.getTagName();
        String name = element_xml.getAttribute("name");
        if (name.length() == 0) {
            ErrorUtils.severe(messageHandler, "_ error.empty.xml.attribute", xpath, "name");
            return;
        }
        xpath = xpath + "[name='" + name + "']";
        if (!StringUtils.isTechnicalName(name, true)) {
            ErrorUtils.severe(messageHandler, "_ error.wrong.xml.attributevalue", xpath, "name", name);
            return;
        }
        if (scrutariSourceDefMap.containsKey(name)) {
            ErrorUtils.severe(messageHandler, "_ error.existing.xml.attributevalue", xpath, "name", name);
            return;
        }
        ScrutariSourceDefBuilder builder = new ScrutariSourceDefBuilder(name);
        boolean done = false;
        NodeList liste = element_xml.getChildNodes();
        for (int i = 0; i < liste.getLength(); i++) {
            Node nd = liste.item(i);
            if (nd.getNodeType() == Node.ELEMENT_NODE) {
                Element element = (Element) liste.item(i);
                String tagName = element.getTagName();
                if (tagName.equals("info-url")) {
                    String urlString = XMLUtils.getData(element);
                    if (urlString.length() == 0) {
                        ErrorUtils.severe(messageHandler, "_ error.empty.xml.tag", xpath + "/" + tagName);
                    } else {
                        try {
                            URL url = new URL(urlString);
                            builder.addInfoUrl(url);
                            done = true;
                        } catch (MalformedURLException mue) {
                            ErrorUtils.severe(messageHandler, "_ error.wrong.xml.tagvalue_url", xpath + "/" + tagName, urlString, mue.getMessage());
                        }
                    }
                } else if (tagName.equals("scrutaridata-url")) {
                    String urlString = XMLUtils.getData(element);
                    if (urlString.length() == 0) {
                        ErrorUtils.severe(messageHandler, "_ error.empty.xml.tag", xpath + "/" + tagName);
                    } else {
                        int frequency = 1;
                        String freqString = element.getAttribute("freq");
                        if (freqString.length() > 0) {
                            try {
                                frequency = Integer.parseInt(freqString);
                            } catch (NumberFormatException nfe) {
                                ErrorUtils.warning(messageHandler, "_ error.wrong.xml.attributevalue_integer", xpath + "/" + tagName, "freq", freqString);
                            }
                        }
                        try {
                            URL url = new URL(urlString);
                            builder.addScrutariDataUrl(url, frequency);
                            done = true;
                        } catch (MalformedURLException mue) {
                            ErrorUtils.severe(messageHandler, "_ error.wrong.xml.tagvalue_url", xpath + "/" + tagName, urlString, mue.getMessage());
                        }
                    }
                } else if (tagName.equals("uri")) {
                    BaseURI baseURI = toBaseURI(element, xpath + "/" + tagName);
                    if (baseURI != null) {
                        builder.setMainBaseURI(baseURI);
                    }
                } else if (tagName.equals("alias")) {
                    BaseURI baseURI = toBaseURI(element, xpath + "/" + tagName);
                    if (baseURI != null) {
                        builder.addAliasBaseURI(baseURI);
                    }
                } else if (tagName.equals("base-icon")) {
                    String baseIcon = XMLUtils.getData(element);
                    if (baseIcon.length() > 0) {
                        builder.setBaseIcon(baseIcon);
                    }
                }
            }
        }
        if (!done) {
            ErrorUtils.severe(messageHandler, "_ error.unsupported.xml.missingtag", xpath, "info-url|scrutaridata-url");
        } else {
            scrutariSourceDefMap.put(name, builder.toScrutariSourceDef());
        }
    }

    private BaseURI toBaseURI(Element element, String xpath) {
        String elString = XMLUtils.getData(element);
        String uriString = elString;
        if (!uriString.startsWith("base:")) {
            uriString = "base:" + uriString;
        }
        try {
            BaseURI baseURI = (BaseURI) URIParser.parse(uriString);
            if (baseURISet.contains(baseURI)) {
                ErrorUtils.severe(messageHandler, "_ error.existing.xml.tagvalue", xpath, elString);
                return null;
            } else {
                baseURISet.add(baseURI);
                return baseURI;
            }
        } catch (URIParseException upe) {
            ErrorUtils.severe(messageHandler, "_ error.wrong.xml.tagvalue_uri", xpath, elString, upe.getMessage());
            return null;
        }
    }

}
