/* SctServer_Directory - Copyright (c) 2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.directory.cache;

import fr.exemole.sctserver.api.storage.SctStorageException;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.io.DataInputPrimitivesReader;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.FicheData;
import net.scrutari.data.IndexationData;
import net.scrutari.data.MotcleData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.db.api.caches.CacheIterator;
import net.scrutari.db.api.caches.DataCacheReader;
import net.scrutari.db.api.caches.DataCacheReaderFactory;


/**
 *
 * @author Vincent Calame
 */
public class DirectoryDataCacheReaderFactory implements DataCacheReaderFactory {

    private final Map<Integer, Long> positionMap;
    private final File[] dataFileArray;

    public DirectoryDataCacheReaderFactory(Map<Integer, Long> positionMap, File[] dataFileArray) {
        this.positionMap = positionMap;
        this.dataFileArray = dataFileArray;
    }

    @Override
    public DataCacheReader newReader() {
        return new InternalDataCacheReader();
    }

    @Override
    public CacheIterator<BaseData> newBaseDataIterator() {
        return CacheUtils.newBaseDataIterator(dataFileArray[CacheUtils.BASE_INDEX]);
    }

    @Override
    public CacheIterator<CorpusData> newCorpusDataIterator() {
        return CacheUtils.newCorpusDataIterator(dataFileArray[CacheUtils.CORPUS_INDEX]);
    }

    @Override
    public CacheIterator<ThesaurusData> newThesaurusDataIterator() {
        return CacheUtils.newThesaurusDataIterator(dataFileArray[CacheUtils.THESAURUS_INDEX]);
    }

    @Override
    public CacheIterator<FicheData> newFicheDataIterator() {
        return CacheUtils.newFicheDataIterator(dataFileArray[CacheUtils.FICHE_INDEX]);
    }

    @Override
    public CacheIterator<MotcleData> newMotcleDataIterator() {
        return CacheUtils.newMotcleDataIterator(dataFileArray[CacheUtils.MOTCLE_INDEX]);
    }

    @Override
    public CacheIterator<IndexationData> newIndexationDataIterator() {
        return CacheUtils.newIndexationDataIterator(dataFileArray[CacheUtils.INDEXATION_INDEX]);
    }

    public static DataCacheReaderFactory loadFromCache(File cacheDirectory) {
        File[] dataFileArray = new File[CacheUtils.LENGTH];
        try {
            dataFileArray[CacheUtils.BASE_INDEX] = testExistence(cacheDirectory, "base");
            dataFileArray[CacheUtils.CORPUS_INDEX] = testExistence(cacheDirectory, "corpus");
            dataFileArray[CacheUtils.THESAURUS_INDEX] = testExistence(cacheDirectory, "thesaurus");
            dataFileArray[CacheUtils.FICHE_INDEX] = testExistence(cacheDirectory, "fiche");
            dataFileArray[CacheUtils.MOTCLE_INDEX] = testExistence(cacheDirectory, "motcle");
            dataFileArray[CacheUtils.INDEXATION_INDEX] = testExistence(cacheDirectory, "indexation");
        } catch (FileNotFoundException ioe) {
            return null;
        }
        Map<Integer, Long> positionMap = new HashMap<Integer, Long>();
        try {
            CacheUtils.checkFichePosition(dataFileArray[CacheUtils.FICHE_INDEX], positionMap);
            CacheUtils.checkMotclePosition(dataFileArray[CacheUtils.MOTCLE_INDEX], positionMap);
        } catch (IOException ioe) {
            return null;
        }
        return new DirectoryDataCacheReaderFactory(positionMap, dataFileArray);
    }

    private static File testExistence(File cacheDirectory, String name) throws FileNotFoundException {
        File f = new File(cacheDirectory, "data-" + name + ".primitives");
        if ((!f.exists()) || (f.isDirectory())) {
            throw new FileNotFoundException(f.getPath());
        }
        return f;
    }


    private class InternalDataCacheReader implements DataCacheReader {

        private final DataReadInfo[] dataReadInfoArray;

        private InternalDataCacheReader() {
            dataReadInfoArray = new DataReadInfo[CacheUtils.LENGTH];
            dataReadInfoArray[CacheUtils.FICHE_INDEX] = initDataReadInfo(CacheUtils.FICHE_INDEX);
            dataReadInfoArray[CacheUtils.MOTCLE_INDEX] = initDataReadInfo(CacheUtils.MOTCLE_INDEX);
        }

        @Override
        public FicheData getFicheData(int codefiche) {
            DataReadInfo dataReadInfo = dataReadInfoArray[CacheUtils.FICHE_INDEX];
            Long position = positionMap.get(codefiche);
            if (position == null) {
                return null;
            }
            try {
                dataReadInfo.dataRandomAccessFile.seek(position);
                return FicheData.fromPrimitives(dataReadInfo.dataPrimitivesReader);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public MotcleData getMotcleData(int codemotcle) {
            DataReadInfo dataReadInfo = dataReadInfoArray[CacheUtils.MOTCLE_INDEX];
            Long position = positionMap.get(codemotcle);
            if (position == null) {
                return null;
            }
            try {
                dataReadInfo.dataRandomAccessFile.seek(position);
                return MotcleData.fromPrimitives(dataReadInfo.dataPrimitivesReader);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public Lang getFicheLang(int codefiche) {
            DataReadInfo dataReadInfo = dataReadInfoArray[CacheUtils.FICHE_INDEX];
            Long position = positionMap.get(codefiche);
            if (position == null) {
                return FicheData.UNDETERMINED_LANG;
            }
            try {
                dataReadInfo.dataRandomAccessFile.seek(position);
                return FicheData.getLangFromPrimitives(dataReadInfo.dataPrimitivesReader);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public int getFicheDateInteger(int codefiche) {
            DataReadInfo dataReadInfo = dataReadInfoArray[CacheUtils.FICHE_INDEX];
            Long position = positionMap.get(codefiche);
            if (position == null) {
                return DateInteger.NULL;
            }
            try {
                dataReadInfo.dataRandomAccessFile.seek(position);
                return FicheData.getDateIntegerFromPrimitives(dataReadInfo.dataPrimitivesReader);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        @Override
        public void close() {
            for (int i = 0; i < CacheUtils.LENGTH; i++) {
                DataReadInfo dataReadInfo = dataReadInfoArray[i];
                if (dataReadInfo != null) {
                    dataReadInfo.close();
                    dataReadInfoArray[i] = null;
                }
            }
        }

        private DataReadInfo initDataReadInfo(int index) {
            try {
                RandomAccessFile dataRandomAccessFile = new RandomAccessFile(dataFileArray[index], "r");
                return new DataReadInfo(dataRandomAccessFile);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

    }

    private static class DataReadInfo {

        private final RandomAccessFile dataRandomAccessFile;
        private final DataInputPrimitivesReader dataPrimitivesReader;

        private DataReadInfo(RandomAccessFile dataRandomAccessFile) {
            this.dataRandomAccessFile = dataRandomAccessFile;
            this.dataPrimitivesReader = new DataInputPrimitivesReader(dataRandomAccessFile);
        }

        private void close() {
            try {
                dataRandomAccessFile.close();
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

    }

}
