/* SctServer_Directory - Copyright (c) 2012-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.directory.cache;

import fr.exemole.sctserver.api.storage.SctStorageException;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.primitives.io.DataOutputPrimitivesWriter;
import net.scrutari.db.api.caches.LexieCacheReaderFactory;
import net.scrutari.db.api.caches.LexieCacheWriter;
import net.scrutari.lexie.FicheLexification;
import net.scrutari.lexie.LexieOccurrencesList;
import net.scrutari.lexie.MotcleLexification;
import net.scrutari.lexie.ScrutariLexieUnitList;


/**
 *
 * @author Vincent Calame
 */
public class DirectoryLexieCacheWriter implements LexieCacheWriter {

    private final File scrutaridbCacheDirectory;
    private final File occurrencesFile;
    private final File lexificationFile;
    private final File unitFile;
    private RandomAccessFile occurrencesRandomAccessFile;
    private DataOutputPrimitivesWriter occurrencesPrimitivesWriter;
    private RandomAccessFile lexificationRandomAccessFile;
    private DataOutputPrimitivesWriter lexificationPrimitivesWriter;
    private RandomAccessFile unitRandomAccessFile;
    private DataOutputPrimitivesWriter unitPrimitivesWriter;
    private Map<Integer, Long> lexificationPositionMap = new HashMap<Integer, Long>();
    private Map<OccurrencesKey, Long> occurrencesPositionMap = new HashMap<OccurrencesKey, Long>();
    private Map<Lang, Long> unitPositionMap = new HashMap<Lang, Long>();

    public DirectoryLexieCacheWriter(File scrutaridbCacheDirectory) {
        this.scrutaridbCacheDirectory = scrutaridbCacheDirectory;
        File occurrences = new File(scrutaridbCacheDirectory, "lexie-occurrences.primitives");
        if (occurrences.exists()) {
            occurrences.delete();
        }
        try {
            this.occurrencesRandomAccessFile = new RandomAccessFile(occurrences, "rw");
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
        this.occurrencesPrimitivesWriter = new DataOutputPrimitivesWriter(occurrencesRandomAccessFile);
        this.occurrencesFile = occurrences;
        File lexification = new File(scrutaridbCacheDirectory, "lexie-lexification.primitives");
        if (lexification.exists()) {
            lexification.delete();
        }
        try {
            this.lexificationRandomAccessFile = new RandomAccessFile(lexification, "rw");
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
        this.lexificationPrimitivesWriter = new DataOutputPrimitivesWriter(lexificationRandomAccessFile);
        this.lexificationFile = lexification;
        File unit = new File(scrutaridbCacheDirectory, "lexie-unit.primitives");
        if (unit.exists()) {
            unit.delete();
        }
        try {
            this.unitRandomAccessFile = new RandomAccessFile(unit, "rw");
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
        this.unitPrimitivesWriter = new DataOutputPrimitivesWriter(unitRandomAccessFile);
        this.unitFile = unit;
    }

    @Override
    public void cacheScrutariLexieUnitList(Lang lang, ScrutariLexieUnitList scrutariLexieUnitList) {
        try {
            unitPrimitivesWriter.writeString(lang.toString());
            long position = unitRandomAccessFile.getFilePointer();
            unitPositionMap.put(lang, position);
            CacheUtils.toPrimitives(scrutariLexieUnitList, unitPrimitivesWriter);
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    @Override
    public void cacheLexieOccurrencesList(long lexieId, int subsetCode, LexieOccurrencesList lexieOccurrencesList) {
        try {
            occurrencesPrimitivesWriter.writeLong(lexieId);
            occurrencesPrimitivesWriter.writeInt(subsetCode);
            long position = occurrencesRandomAccessFile.getFilePointer();
            occurrencesPositionMap.put(new OccurrencesKey(lexieId, subsetCode), position);
            CacheUtils.toPrimitives(lexieOccurrencesList, occurrencesPrimitivesWriter);
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    @Override
    public void cacheFicheLexification(int codefiche, FicheLexification ficheLexification) {
        try {
            lexificationPrimitivesWriter.writeInt(codefiche);
            lexificationPrimitivesWriter.writeChar('f');
            long position = lexificationRandomAccessFile.getFilePointer();
            lexificationPositionMap.put(codefiche, position);
            FicheLexification.toPrimitives(ficheLexification, lexificationPrimitivesWriter);
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    @Override
    public void cacheMotcleLexification(int codemotcle, MotcleLexification motcleLexification) {
        try {
            lexificationPrimitivesWriter.writeInt(codemotcle);
            lexificationPrimitivesWriter.writeChar('m');
            long position = lexificationRandomAccessFile.getFilePointer();
            lexificationPositionMap.put(codemotcle, position);
            MotcleLexification.toPrimitives(motcleLexification, lexificationPrimitivesWriter);
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    @Override
    public LexieCacheReaderFactory endCacheWrite() {
        try {
            occurrencesPrimitivesWriter = null;
            occurrencesRandomAccessFile.close();
            occurrencesRandomAccessFile = null;
            lexificationPrimitivesWriter = null;
            lexificationRandomAccessFile.close();
            lexificationRandomAccessFile = null;
            unitPrimitivesWriter = null;
            unitRandomAccessFile.close();
            unitRandomAccessFile = null;
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
        CacheUtils.saveCacheVersion(scrutaridbCacheDirectory);
        return new DirectoryLexieCacheReaderFactory(occurrencesFile, lexificationFile, unitFile, lexificationPositionMap, occurrencesPositionMap, unitPositionMap);
    }

}
