/* UtilLib - Copyright (c) 2012-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.primitives;

import java.text.ParseException;
import java.util.Collection;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class DateIntegerFilterUtils {

    public final static DateIntegerFilter NULL_FILTER = new NullDateIntegerFilter(true);
    public final static DateIntegerFilter NOTNULL_FILTER = new NullDateIntegerFilter(false);

    private DateIntegerFilterUtils() {
    }

    public static DateIntegerFilterList toDateIntegerFilterList(Collection<DateIntegerFilter> collection) {
        int size = collection.size();
        return new ArrayDateIntegerFilterList(collection.toArray(new DateIntegerFilter[size]));
    }

    public static RangeDateIntegerFilter parseDateIntegerFilter(String s) throws ParseException {
        String[] tokens = StringUtils.getTechnicalTokens(s, false);
        int length = tokens.length;
        if (length == 0) {
            return null;
        }
        if (length > 2) {
            throw new ParseException("Too many tokens", s.length() - 1);
        }
        if (length == 1) {
            try {
                int dateInteger = DateInteger.parse(tokens[0]);
                return DateIntegerFilterFactory.newInstance(dateInteger);
            } catch (DateIntegerException die) {
                throw new ParseException("Wrong date: " + tokens[0], 0);
            }
        } else {
            int date1;
            int date2;
            try {
                date1 = DateInteger.parse(tokens[0]);
            } catch (DateIntegerException die) {
                throw new ParseException("Wrong date: " + tokens[0], 0);
            }
            try {
                date2 = DateInteger.parse(tokens[1]);
            } catch (DateIntegerException die) {
                throw new ParseException("Wrong date: " + tokens[1], 0);
            }
            return DateIntegerFilterFactory.newInstance(date1, date2);
        }
    }

    public static String toString(DateIntegerFilter dateIntegerFilter) {
        if (dateIntegerFilter instanceof RangeDateIntegerFilter) {
            RangeDateIntegerFilter rangeDateIntegerFilter = (RangeDateIntegerFilter) dateIntegerFilter;
            int min = rangeDateIntegerFilter.getMinDateInteger();
            int max = rangeDateIntegerFilter.getMaxDateInteger();
            if (min == max) {
                return DateInteger.toString(min);
            } else {
                return DateInteger.toString(min) + " ; " + DateInteger.toString(max);
            }
        }
        return "";
    }

    public static boolean intersects(int dateInteger, DateIntegerFilter dateIntegerFilter) {
        if (dateIntegerFilter.containsDateInteger(dateInteger)) {
            return true;
        }
        if (!(dateIntegerFilter instanceof RangeDateIntegerFilter)) {
            return false;
        }
        return intersects((RangeDateIntegerFilter) dateIntegerFilter, DateIntegerFilterFactory.newInstance(dateInteger));
    }

    public static boolean intersects(int date1, int date2, DateIntegerFilter dateIntegerFilter) {
        if (dateIntegerFilter.containsDateInteger(date1)) {
            return true;
        }
        if (dateIntegerFilter.containsDateInteger(date2)) {
            return true;
        }
        if (!(dateIntegerFilter instanceof RangeDateIntegerFilter)) {
            return false;
        }
        return intersects((RangeDateIntegerFilter) dateIntegerFilter, DateIntegerFilterFactory.newInstance(date1, date2));
    }

    private static boolean intersects(RangeDateIntegerFilter rangeDateIntegerFilter, DateIntegerFilter otherFilter) {
        int min = rangeDateIntegerFilter.getMinDateInteger();
        int max = rangeDateIntegerFilter.getMaxDateInteger();
        if (min == max) {
            return otherFilter.containsDateInteger(min);
        } else {
            return (otherFilter.containsDateInteger(min) || otherFilter.containsDateInteger(max));
        }
    }


    private static class ArrayDateIntegerFilterList implements DateIntegerFilterList {

        private DateIntegerFilter[] array;

        private ArrayDateIntegerFilterList(DateIntegerFilter[] array) {
            this.array = array;
        }

        public int getDateIntegerFilterCount() {
            return array.length;
        }

        public DateIntegerFilter getDateIntegerFilter(int i) {
            return array[i];
        }

    }


    private static class NullDateIntegerFilter implements DateIntegerFilter {

        private boolean isNull;

        private NullDateIntegerFilter(boolean isNull) {
            this.isNull = isNull;
        }

        public boolean containsDateInteger(int dateInteger) {
            if (dateInteger == DateInteger.NULL) {
                return isNull;
            } else {
                return !isNull;
            }
        }

    }

}
