/* UtilLib  - Copyright (c) 2012-2013 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.text.gabarit;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.exceptions.ShouldNotOccurException;


/**
 * Dans un fichier Texte, un élément d'inclusion se trouve avec l'expression
 * rationnelle suivante : \$[a-zA-Z][_a-zA-Z0-9]*(:[-_,;/.|a-zA-Z0-9]*)*\$
 *
 * @author Vincent Calame
 */
public final class GabaritFactory {

    private GabaritFactory() {
    }

    public static Gabarit parse(String gabaritString) {
        try {
            return parse(new StringReader(gabaritString));
        } catch (IOException ioe) {
            throw new ShouldNotOccurException(ioe);
        }
    }

    public static Gabarit parse(Reader gabaritReader) throws IOException {
        BufferedReader bufreader = new BufferedReader(gabaritReader);
        InternalGabaritBuilder builder = new InternalGabaritBuilder();
        String ligne;
        while ((ligne = bufreader.readLine()) != null) {
            builder.testNewLine(ligne);
        }
        return builder.toGabarit();
    }


    private static class InternalGabaritBuilder {

        private final List<Object> objectList = new ArrayList<Object>();
        private StringBuilder currentStringBuilder = new StringBuilder();
        private boolean firstLine = true;

        private void testNewLine(String line) {
            if (firstLine) {
                firstLine = false;
            } else {
                currentStringBuilder.append('\n');
            }
            int openIndex = line.indexOf('$');
            if (openIndex == -1) {
                currentStringBuilder.append(line);
                return;
            }
            int closeIndex = line.indexOf('$', openIndex + 1);
            if (closeIndex == -1) {
                currentStringBuilder.append(line);
                return;
            }
            InternalGabaritInclusion gabaritInclusion = parseInclusion(line, openIndex + 1, closeIndex);
            if (gabaritInclusion != null) {
                currentStringBuilder.append(line, 0, openIndex);
                flush();
                objectList.add(gabaritInclusion);
                testRemainInLine(line, closeIndex + 1);
            } else {
                currentStringBuilder.append(line, 0, closeIndex);
                testRemainInLine(line, closeIndex);
            }
        }

        private void testRemainInLine(String line, int startIndex) {
            int openIndex = line.indexOf('$', startIndex);
            if (openIndex == -1) {
                currentStringBuilder.append(line, startIndex, line.length());
                return;
            }
            int closeIndex = line.indexOf('$', openIndex + 1);
            if (closeIndex == -1) {
                currentStringBuilder.append(line, startIndex, line.length());
                return;
            }
            InternalGabaritInclusion gabaritInclusion = parseInclusion(line, openIndex + 1, closeIndex);
            if (gabaritInclusion != null) {
                currentStringBuilder.append(line, startIndex, openIndex);
                flush();
                objectList.add(gabaritInclusion);
                testRemainInLine(line, closeIndex + 1);
            } else {
                currentStringBuilder.append(line, 0, closeIndex);
                testRemainInLine(line, closeIndex);
            }
        }

        private void flush() {
            if (currentStringBuilder.length() > 0) {
                objectList.add(currentStringBuilder.toString());
                currentStringBuilder = new StringBuilder();
            }
        }

        private InternalGabaritInclusion parseInclusion(String s, int startIndex, int endIndex) {
            if (startIndex == endIndex) {
                return null;
            }
            char firstChar = s.charAt(startIndex);
            if (!testFirstChar(firstChar)) {
                return null;
            }
            StringBuilder nameBuilder = new StringBuilder();
            nameBuilder.append(firstChar);
            int argumentIndex = -1;
            for (int i = (startIndex + 1); i < endIndex; i++) {
                char c = s.charAt(i);
                if (c == ':') {
                    argumentIndex = i + 1;
                    break;
                }
                if (!testNameChar(c)) {
                    return null;
                }
                nameBuilder.append(c);
            }
            InternalGabaritInclusion gabaritInclusion = new InternalGabaritInclusion(nameBuilder.toString());
            while (argumentIndex != -1) {
                StringBuilder argumentBuilder = new StringBuilder();
                boolean newArgument = false;
                for (int i = argumentIndex; i < endIndex; i++) {
                    char c = s.charAt(i);
                    if (c == ':') {
                        argumentIndex = i + 1;
                        newArgument = true;
                        break;
                    }
                    if (!testArgumentChar(c)) {
                        return null;
                    }
                    argumentBuilder.append(c);
                }
                gabaritInclusion.addArgument(argumentBuilder.toString());
                if (!newArgument) {
                    break;
                }
            }
            return gabaritInclusion;
        }

        private Gabarit toGabarit() {
            flush();
            Object[] gabaritElementArray = new Object[objectList.size()];
            return new InternalGabarit(objectList.toArray(gabaritElementArray));
        }

    }


    private static class InternalGabarit implements Gabarit {

        private final Object[] gabaritElementArray;

        private InternalGabarit(Object[] gabaritElementArray) {
            this.gabaritElementArray = gabaritElementArray;
        }

        @Override
        public int getGabaritElementCount() {
            return gabaritElementArray.length;
        }

        @Override
        public Object getGabaritElement(int i) {
            return gabaritElementArray[i];
        }

    }


    private static class InternalGabaritInclusion implements GabaritInclusion {

        private final String name;
        private final List<String> argumentList = new ArrayList<String>();

        private InternalGabaritInclusion(String name) {
            this.name = name;
        }

        @Override
        public String getName() {
            return name;
        }

        @Override
        public int getArgumentCount() {
            return argumentList.size();
        }

        @Override
        public String getArgument(int i) {
            return argumentList.get(i);
        }

        private void addArgument(String s) {
            argumentList.add(s);
        }

    }

    private static boolean testFirstChar(char c) {
        if ((c >= 'A') && (c <= 'Z')) {
            return true;
        }
        if ((c >= 'a') && (c <= 'z')) {
            return true;
        }
        return false;
    }

    private static boolean testNameChar(char c) {
        if (c == '_') {
            return true;
        }
        if ((c >= 'A') && (c <= 'Z')) {
            return true;
        }
        if ((c >= 'a') && (c <= 'z')) {
            return true;
        }
        if ((c >= '0') && (c <= '9')) {
            return true;
        }
        return false;
    }

    private static boolean testArgumentChar(char c) {
        if ((c >= 'A') && (c <= 'Z')) {
            return true;
        }
        if ((c >= 'a') && (c <= 'z')) {
            return true;
        }
        if ((c >= '0') && (c <= '9')) {
            return true;
        }
        switch (c) {
            case '_':
            case '/':
            case '-':
            case '.':
            case ',':
            case ';':
            case '|':
                return true;
            default:
                return false;
        }
    }

}
